# $Id: test_MurckoScaffold.py 3672 2010-06-14 17:10:00Z landrgr1 $
#
# Created by Peter Gedeck, June 2008
#

import doctest
import unittest
from collections import namedtuple

from rdkit import Chem
from rdkit.Chem.Scaffolds import MurckoScaffold
from rdkit.Chem.Scaffolds.MurckoScaffold import (
    GetScaffoldForMol, MakeScaffoldGeneric, MurckoScaffoldSmiles,
    MurckoScaffoldSmilesFromSmiles, _pyGetScaffoldForMol)

TestMolecule = namedtuple('TestMolecule', 'smiles,scaffold')


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(MurckoScaffold, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def test1MurckoScaffold(self):
    # Test the functionality on a smaller test set
    for testMol in self.testMolecules:
      mol = Chem.MolFromSmiles(testMol.smiles)
      calcScaffold = Chem.MolToSmiles(GetScaffoldForMol(mol))
      actualScaffold = Chem.MolToSmiles(Chem.MolFromSmiles(testMol.scaffold))
      self.assertEqual(calcScaffold, actualScaffold)

  def test2MurckoScaffold(self):
    # Test the functionality on a larger test set
    for testMol in self.testMolecules2:
      mol = Chem.MolFromSmiles(testMol.smiles)
      calcScaffold = Chem.MolToSmiles(GetScaffoldForMol(mol))
      actualScaffold = Chem.MolToSmiles(Chem.MolFromSmiles(testMol.scaffold))
      self.assertEqual(calcScaffold, actualScaffold)

  def test_ReferenceImplementation(self):
    # Check that the C++ implementation is equivalent to the Python reference implementation
    for testMol in self.testMolecules:
      mol = Chem.MolFromSmiles(testMol.smiles)
      calcScaffold1 = Chem.MolToSmiles(GetScaffoldForMol(mol))
      calcScaffold2 = Chem.MolToSmiles(_pyGetScaffoldForMol(mol))
      self.assertEqual(calcScaffold1, calcScaffold2)

  def test_MurckScaffoldSmilesFromSmiles(self):
    self.assertEqual(MurckoScaffoldSmilesFromSmiles('Cc1cc(Oc2nccc(CCC)c2)ccc1'),
                     'c1ccc(Oc2ccccn2)cc1')
    self.assertEqual(MurckoScaffoldSmilesFromSmiles('CCCC'), '')

  def test_MurckoScaffoldSmiles(self):
    self.assertEqual(MurckoScaffoldSmiles('Cc1cc(Oc2nccc(CCC)c2)ccc1'), 'c1ccc(Oc2ccccn2)cc1')
    self.assertEqual(MurckoScaffoldSmiles(mol=Chem.MolFromSmiles('Cc1cc(Oc2nccc(CCC)c2)ccc1')),
                     'c1ccc(Oc2ccccn2)cc1')
    self.assertRaises(ValueError, MurckoScaffoldSmiles, smiles=None, mol=None)

  def test_MakeScaffoldGeneric(self):

    def testSmiles(smiles):
      return Chem.MolToSmiles(MakeScaffoldGeneric(Chem.MolFromSmiles(smiles)))

    self.assertEqual(testSmiles('c1ccccc1'), 'C1CCCCC1')
    self.assertEqual(testSmiles('c1cccnc1'), 'C1CCCCC1')

    # Examples associated with sf.net issue 246
    self.assertEqual(testSmiles('c1[nH]ccc1'), 'C1CCCC1')
    self.assertEqual(testSmiles('C1[NH2+]C1'), 'C1CC1')
    self.assertEqual(testSmiles('C1[C@](Cl)(F)O1'), 'CC1(C)CC1')
    
    # Examples from github issue 6855
    self.assertEqual(testSmiles('[235U]1CC1'), 'C1CC1')

  testMolecules = [
    TestMolecule('CC1CCC1', 'C1CCC1'),
    TestMolecule('NCNCC2CC2C1CC1O', 'C1CC1C1CC1'),
    # Spiro
    TestMolecule('OC2C(C)C21C(N)C1C', 'C2CC12CC1'),
    # Carbonyl outside scaffold
    TestMolecule('C1CC1C(=O)OC', 'C1CC1'),
    # Double bond outside scaffold
    TestMolecule('C1CC1C=C', 'C1CC1'),
    # Double bond in scaffold
    TestMolecule('C1CC1C=CC1CC1C=CNNCO', 'C1CC1C=CC1CC1'),
    TestMolecule('CC1CC1C(N)C1C(N)C1', 'C1CC1CC1CC1'),
    # Double bond in linker
    TestMolecule('C1CC1C(C(C)C)=NC1CC1', 'C1CC1C=NC1CC1'),
    # S=O group in scaffold
    TestMolecule('C1CC1S(=O)C1CC1C=CNNCO', 'C1CC1S(=O)C1CC1'),
    # S=O group outside scaffold
    TestMolecule('O=SCNC1CC1S(=O)C1CC1C=CNNCO', 'C1CC1S(=O)C1CC1'),
    # SO2 group in scaffold
    TestMolecule('C1CC1S(=O)(=O)C1CC1C=CNNCO', 'C1CC1S(=O)(=O)C1CC1'),
    # SO2 group outside scaffold
    TestMolecule('O=S(CNCNC)(=O)CNC1CC1S(=O)(=O)C1CC1C=CNNCO', 'C1CC1S(=O)(=O)C1CC1'),
    # Hydroxamide
    TestMolecule('C1CC1C=NO', 'C1CC1'),
    # Cyano group
    TestMolecule('C1CC1C#N', 'C1CC1'),
    # Acetylene group
    TestMolecule('C1CC1C#CNC', 'C1CC1'),
    TestMolecule('O=C1N(C)C(=O)N1C#CNC', 'O=C1NC(=O)N1'),
    TestMolecule('[O-][N+](=O)c1cc(ccc1Cl)NS(=O)(=O)Cc2ccccc2', 'c1ccccc1NS(=O)(=O)Cc2ccccc2'),
    # N-Substituted pyrrol
    TestMolecule('Cn1cccc1', 'c1ccc[nH]1'),
    # Explicit hydrogens are removed
    TestMolecule('C1CC1[CH](C)C1CC1', 'C1CC1CC1CC1'),
  ]

  testMolecules2 = [
    TestMolecule('CCOc1ccccc1N(S(C)(=O)=O)CC(NC1CCCCC1)=O', 'O=C(NC1CCCCC1)CNc1ccccc1'),
    TestMolecule('c1ccc(-c2c(C)n(-c3c(C(O)=O)cccc3)c(C)nc2=O)cc1',
                 'O=c1c(cn(cn1)-c1ccccc1)-c1ccccc1'),
    TestMolecule('Cc1ccc(Cl)c2c1NC(=O)C2=C1NC(=S)NC1=O', 'c1cc2c(cc1)C(=C1C(NC(N1)=S)=O)C(=O)N2'),
    TestMolecule('CNC(=O)CCc1[nH]c2c(c1Sc1ccccc1)cccc2', 'c1cc(Sc2c3c([nH]c2)cccc3)ccc1'),
    TestMolecule('CC(=O)OCC(=O)C1(O)CCC2C1(C)CC(=O)C1C3(C)CCC(=O)C=C3CCC21',
                 'O=C1C=C2CCC3C4CCCC4CC(=O)C3C2CC1'),
    TestMolecule('CC(C)CC(Nc1nc(Cl)ccc1[N+]([O-])=O)C(O)=O', 'c1ccncc1'),
    TestMolecule('COc1ccc(C(Nc2ccc(S(N3C(C)CCCC3)(=O)=O)cc2)=O)c(OC)c1OC',
                 'O=C(Nc1ccc(S(=O)(=O)N2CCCCC2)cc1)c1ccccc1'),
    TestMolecule('CC(C)CCNc1nc(N)c([N+](=O)[O-])c(NCCO)n1', 'c1cncnc1'),
    TestMolecule('c1ccc(Oc2c(NC(COC(c3c(C)noc3C)=O)=O)cccc2)cc1',
                 'O=C(COC(=O)c1cnoc1)Nc1ccccc1Oc1ccccc1'),
    TestMolecule('COC(CCCCC1SCC(NC(OC)=O)C1NC(OC)=O)=O', 'C1CCCS1'),
    TestMolecule('CSc1ccc(-c2c(C#N)c(N)nc3n(-c4ccccc4)nc(C)c32)cc1',
                 'c1ccc(cc1)-c1c2c(n(nc2)-c2ccccc2)ncc1'),
    TestMolecule('O=C1Cc2ccccc2Sc2c1cc(Cl)cc2', 'O=C1Cc2ccccc2Sc2ccccc21'),
    TestMolecule('COC(c1n(CC(N(C)c2ccccc2)=O)c2ccsc2c1)=O', 'O=C(Cn1c2ccsc2cc1)Nc1ccccc1'),
    TestMolecule('N=C1C(=Cc2coc3ccccc3c2=O)C(=O)N=C2SC(c3ccncc3)=NN12',
                 'N=C1C(=Cc2coc3ccccc3c2=O)C(=O)N=C2SC(c3ccncc3)=NN12'),
    TestMolecule('CCOC(c1ccc(NC(CCc2c(C)nc3ncnn3c2C)=O)cc1)=O', 'O=C(Nc1ccccc1)CCc1cnc2n(ncn2)c1'),
    TestMolecule('COC(=O)C1=C(C)NC(C)=C(C(OC)=O)C1c1oc(-c2c(Cl)c(Cl)ccc2)cc1',
                 'c1ccc(-c2oc(C3C=CNC=C3)cc2)cc1'),
    TestMolecule('CCN(S(c1cc(NC(COC(CCc2nc3ccccc3s2)=O)=O)ccc1)(=O)=O)CC',
                 'c1cc(NC(COC(=O)CCc2nc3c(s2)cccc3)=O)ccc1'),
    TestMolecule('CCOC(c1cc(OC(c2ccccc2)=O)n(-c2ccccc2)n1)=O', 'O=C(Oc1n(ncc1)-c1ccccc1)c1ccccc1'),
    TestMolecule('CCOC(=O)c1nc2c(c(NCc3ccccc3F)n1)cccc2', 'c1ccc(CNc2ncnc3c2cccc3)cc1'),
    TestMolecule('Cc1nc(C)n(CC(N2CCCC(C(c3c(C)cc(Cl)cc3)=O)C2)=O)n1',
                 'c1ccc(cc1)C(=O)C1CCCN(C(=O)Cn2cncn2)C1'),
    TestMolecule('COc1cc(NC(=O)c2nnn(CCc3ccccc3)c2N)c(OC)cc1', 'O=C(c1nnn(c1)CCc1ccccc1)Nc1ccccc1'),
    TestMolecule('Cc1cc(C(=O)CN2C(=O)c3ccccc3C2=O)c(C)n1Cc1cccs1',
                 'O=C(CN1C(c2c(cccc2)C1=O)=O)c1cn(Cc2cccs2)cc1'),
    TestMolecule('c1cnc2c(c1)cccc2S(N1CCC(C(=O)N2CCN(c3ccc(Cl)cc3)CC2)CC1)(=O)=O',
                 'c1ccc(cc1)N1CCN(C(=O)C2CCN(S(=O)(=O)c3c4ncccc4ccc3)CC2)CC1'),
    TestMolecule('CCOC(c1c(C)[nH]c(C(NNC(c2ccc(C(C)(C)C)cc2)=O)=O)c1C)=O',
                 'c1ccc(cc1)C(NNC(c1ccc[nH]1)=O)=O'),
    TestMolecule('CCOC(c1cc(C(C)C)sc1NC(=O)COC(CCS(c1ccccc1)(=O)=O)=O)=O',
                 'c1ccc(S(CCC(=O)OCC(Nc2cccs2)=O)(=O)=O)cc1'),
    TestMolecule('CCC1CCCCN1CCCNC(=O)Cn1nc(-c2ccccc2)ccc1=O',
                 'O=C(NCCCN1CCCCC1)Cn1nc(ccc1=O)-c1ccccc1'),
    TestMolecule('CCc1cc(OCCn2nc(C(O)=O)c3ccccc3c2=O)ccc1', 'O=c1n(CCOc2ccccc2)ncc2ccccc21'),
    TestMolecule('Fc1ccc(CN2CCN3C(CCC3)C2C2CCCCC2)cc1F', 'c1ccc(cc1)CN1CCN2CCCC2C1C1CCCCC1'),
    TestMolecule('O=[N+]([O-])c1cc(-c2nnc(N3CCOCC3)c3ccccc23)ccc1N1CCOCC1',
                 'c1cc2c(nnc(c2cc1)N1CCOCC1)-c1ccc(cc1)N1CCOCC1'),
    TestMolecule('Cc1ccnc(NC(=O)COc2ccc3oc4c(c3c2)CCCC4)c1',
                 'O=C(COc1ccc2oc3c(c2c1)CCCC3)Nc1ccccn1'),
    TestMolecule('Cc1cc(=O)oc(C)c1C(=O)NCCCN1CCN(c2ccc(F)cc2)CC1',
                 'c1ccc(N2CCN(CCCNC(c3ccc(oc3)=O)=O)CC2)cc1'),
    TestMolecule('Cc1cc(C(=O)CSc2nc(=O)cc(N)[nH]2)c(C)n1-c1cccc(F)c1',
                 'O=C(CSc1nc(cc[nH]1)=O)c1cn(cc1)-c1ccccc1'),
    TestMolecule('CCN(S(c1cccc(C(=O)N2CCCCC2)c1)(=O)=O)CC', 'O=C(N1CCCCC1)c1ccccc1'),
    TestMolecule('CNC(=S)N1CCC(NC(=O)C23CC4CC(C2)CC(C3)C4)CC1',
                 'O=C(NC1CCNCC1)C12CC3CC(C1)CC(C3)C2'),
    TestMolecule('Cc1cc2c(cc1)N=C(C)C(N=O)=C(C)N2', 'c1cc2NC=CC=Nc2cc1'),
    TestMolecule('COc1ccc(Sc2cc(C(F)(F)F)nc(-c3ncccc3)n2)cc1', 'c1ccc(cc1)Sc1nc(ncc1)-c1ncccc1'),
    TestMolecule('c1coc(CNC(Cn2cc(C(c3ccccc3)=O)c3c2cccc3)=O)c1',
                 'c1coc(CNC(Cn2cc(C(c3ccccc3)=O)c3c2cccc3)=O)c1'),
    TestMolecule('O=C(NCc1ccc(Cl)cc1)c1noc(-c2ccco2)c1', 'O=C(c1noc(c1)-c1ccco1)NCc1ccccc1'),
    TestMolecule('CN(C)c1ccc(C(c2n(CCOC(=O)Nc3ccc(Cl)cc3)nnn2)N2CCOCC2)cc1',
                 'O=C(Nc1ccccc1)OCCn1nnnc1C(c1ccccc1)N1CCOCC1'),
    TestMolecule('NC(=NOC(=O)c1cc(Cn2cc(C(F)(F)F)ccc2=O)ccc1)c1ccccc1',
                 'c1ccc(C=NOC(c2cc(Cn3ccccc3=O)ccc2)=O)cc1'),
    TestMolecule('CCc1nnc(NC(=O)Cc2c(-c3ccc(C)cc3)nc(C)s2)s1', 'O=C(Cc1c(-c2ccccc2)ncs1)Nc1nncs1'),
    TestMolecule('COCCCNC(=O)CN1C(=O)N(Cc2ccccc2Cl)CC1', 'O=C1NCCN1Cc1ccccc1'),
    TestMolecule('Cc1cc([N+]([O-])=O)nn1CC(=O)NCCCn1ccnc1', 'O=C(Cn1nccc1)NCCCn1ccnc1'),
    TestMolecule('c1cc(F)c(N2CCN(C(=O)c3ccc(S(NCC4OCCC4)(=O)=O)cc3)CC2)cc1',
                 'c1ccc(cc1)N1CCN(C(c2ccc(cc2)S(=O)(=O)NCC2OCCC2)=O)CC1'),
    TestMolecule('CC(NCc1cccnc1)=C1C(=O)NC(=O)N(c2ccc(C)cc2)C1=O',
                 'c1cc(ccc1)N1C(=O)NC(C(=CNCc2cccnc2)C1=O)=O'),
    TestMolecule('Cc1ccn(C)c(=N)c1', 'N=c1[nH]cccc1'),
    TestMolecule('Cc1cc(C)nc(N2CCC(CNC(=O)CCc3ccccc3)CC2)n1',
                 'O=C(CCc1ccccc1)NCC1CCN(c2ncccn2)CC1'),
    TestMolecule('CCOC1=CC(=CNNC(CCCC(NC2CCCCC2)=O)=O)C=CC1=O',
                 'C1=CC(C=CC1=O)=CNNC(=O)CCCC(=O)NC1CCCCC1'),
    TestMolecule('CC(=O)N1CCN(c2ccc([N+]([O-])=O)cc2)CC1', 'c1ccc(cc1)N1CCNCC1'),
    TestMolecule('CS(N(CC(=O)N1CCCCC1)Cc1ccc(Cl)cc1)(=O)=O', 'O=C(N1CCCCC1)CNCc1ccccc1'),
    TestMolecule('c1coc(C(=O)N2CCN(C(COc3cc(C(NCc4ccccc4)=O)ccc3)=O)CC2)c1',
                 'c1coc(C(=O)N2CCN(C(COc3cc(C(NCc4ccccc4)=O)ccc3)=O)CC2)c1'),
    TestMolecule('Cc1cccc2sc(NNC(=O)C3=COCCO3)nc12', 'O=C(NNc1nc2ccccc2s1)C1=COCCO1'),
    TestMolecule('c1ccc2c(c1)N(C)C1(C=Nc3c(cc(N4CCOCC4)c4ccccc34)O1)C2(C)C',
                 'C1=Nc2c(cc(c3ccccc23)N2CCOCC2)OC11Nc2ccccc2C1'),
    TestMolecule('COc1cccc(C2N(CCN3CCOCC3)C(=O)C(O)=C2C(=O)c2sc(C)nc2C)c1',
                 'O=C(C1=CC(=O)N(C1c1ccccc1)CCN1CCOCC1)c1scnc1'),
    TestMolecule('COc1cc(OC)c(NC(CSc2nc3c(c(=O)n2-c2ccc(F)cc2)SCC3)=O)cc1',
                 'c1ccc(cc1)NC(=O)CSc1n(c(=O)c2c(n1)CCS2)-c1ccccc1'),
    TestMolecule('Cc1ccccc1CN1c2ccccc2C2(C1=O)OCCCO2', 'O=C1C2(OCCCO2)c2c(N1Cc1ccccc1)cccc2'),
    TestMolecule('O=C(N1C2(OCC1)CCN(c1ncc(C(F)(F)F)cc1Cl)CC2)c1ccccc1',
                 'O=C(c1ccccc1)N1C2(OCC1)CCN(c1ccccn1)CC2'),
    TestMolecule('CC=CC=CC(=O)Nc1nccs1', 'c1ncsc1'),
    TestMolecule('CC(C)(C)c1ccc(C(c2c[nH]c(C(NCc3cccnc3)=O)c2)=O)cc1',
                 'c1ccc(cc1)C(=O)c1c[nH]c(c1)C(=O)NCc1cccnc1'),
    TestMolecule('CCC(=O)Nc1c(C)nn(-c2cc(C)c(C)cc2)c1C', 'c1ccc(cc1)-n1nccc1'),
    TestMolecule('Cc1ccc(SCCC(=O)NCCSCc2c(C)cccc2)cc1', 'O=C(NCCSCc1ccccc1)CCSc1ccccc1'),
    TestMolecule('CC1=NN(Cc2ccccc2)C(=O)C1=Cc1ccc(N(C)C)cc1', 'O=C1C(C=NN1Cc1ccccc1)=Cc1ccccc1'),
    TestMolecule('COCC(=O)Nc1ccc(S(NCCc2ccccc2)(=O)=O)cc1', 'c1ccc(CCNS(=O)(=O)c2ccccc2)cc1'),
    TestMolecule('CCOC(=O)N(C)c1ccc(C(O)(C(F)(F)F)C(F)(F)F)cc1', 'c1ccccc1'),
    TestMolecule('Fc1ccc(COC2=C(C(O)=O)CCNC2=O)cc1F', 'O=C1NCCC=C1OCc1ccccc1'),
    TestMolecule('O=C1N2C(Nc3ccccc31)CCCCC2', 'O=C1N2C(Nc3ccccc31)CCCCC2'),
    TestMolecule('Cl.COc1ccc(-c2nc3n(ccc4ccccc43)c2CN2CCOCC2)cc1OC',
                 'c1cccc(c1)-c1nc2c3c(ccn2c1CN1CCOCC1)cccc3'),
    TestMolecule('ClCc1oc(-c2ccccc2)nn1', 'c1oc(nn1)-c1ccccc1'),
    TestMolecule('Cl.Cc1ccc(OCC(O)Cn2c(=N)n(CCN3CCCCC3)c3ccccc32)cc1',
                 'N=c1n(CCCOc2ccccc2)c2ccccc2n1CCN1CCCCC1'),
    TestMolecule('COc1ccc(C(=O)C=C(C)Nc2ccc3c(c2)OCO3)cc1', 'O=C(C=CNc1ccc2c(c1)OCO2)c1ccccc1'),
    TestMolecule('c1csc(CN(C(c2ccc(F)cc2)C(NC2CCCCC2)=O)C(=O)CN2S(=O)(=O)c3ccccc3C2=O)c1',
                 'c1cc(CN(C(=O)CN2S(=O)(c3ccccc3C2=O)=O)C(C(=O)NC2CCCCC2)c2ccccc2)sc1'),
    TestMolecule('c1csc(S(NCCSc2n(-c3ccccc3)nnn2)(=O)=O)c1',
                 'c1csc(S(NCCSc2n(-c3ccccc3)nnn2)(=O)=O)c1'),
    TestMolecule('Cc1cccc(C=NNC(=O)Cn2c(N)nnn2)n1', 'O=C(Cn1cnnn1)NN=Cc1ccccn1'),
    TestMolecule('CCOC(C1(Cc2ccc(Cl)cc2)CCN(C(c2cc(C)nc(C)n2)=O)CC1)=O',
                 'O=C(N1CCC(CC1)Cc1ccccc1)c1ccncn1'),
    TestMolecule('c1ccc(C(N(CC2OCCC2)C(Cn2nnc3ccccc23)=O)C(NCc2ccc(F)cc2)=O)cc1',
                 'O=C(N(C(c1ccccc1)C(=O)NCc1ccccc1)CC1OCCC1)Cn1nnc2c1cccc2'),
    TestMolecule('O=C1CSC(c2ccncc2)N1Cc1occc1', 'O=C1CSC(c2ccncc2)N1Cc1occc1'),
    TestMolecule('COc1c(OCc2ccccc2)c(Br)cc(C=NNC(=O)Cn2nc([N+]([O-])=O)cc2C)c1',
                 'O=C(Cn1nccc1)NN=Cc1ccc(cc1)OCc1ccccc1'),
    TestMolecule('Cc1c(Cn2nnc(-c3cc(C(=O)O)ccc3)n2)cccc1', 'c1cccc(-c2nn(nn2)Cc2ccccc2)c1'),
    TestMolecule('O=C(c1ccc2snnc2c1)N1CCCC1', 'O=C(c1ccc2snnc2c1)N1CCCC1'),
    TestMolecule('c1ccc(CC(NN2C(=O)C(=Cc3c(C(O)=O)cccc3)SC2=S)=O)cc1',
                 'O=C1C(=Cc2ccccc2)SC(=S)N1NC(Cc1ccccc1)=O'),
    TestMolecule('Cc1ccccc1OCC(=O)NN=Cc1ccncc1', 'O=C(COc1ccccc1)NN=Cc1ccncc1'),
    TestMolecule('O=C(C=Cc1ccccc1)NC(=S)Nc1ccc(CN2CCOCC2)cc1',
                 'O=C(C=Cc1ccccc1)NC(=S)Nc1ccc(CN2CCOCC2)cc1'),
    TestMolecule('COc1ccc(NC(=S)N(Cc2cnccc2)Cc2c(=O)[nH]c3c(c2)cc(OC)c(OC)c3)cc1',
                 'O=c1c(CN(C(=S)Nc2ccccc2)Cc2cnccc2)cc2ccccc2[nH]1'),
    TestMolecule('Nc1ccc2nc3c([nH]c(=O)n(C4CCCCC4)c3=O)nc2c1',
                 'c1ccc2nc3[nH]c(n(c(c3nc2c1)=O)C1CCCCC1)=O'),
    TestMolecule('Cc1cc(NC(=O)c2ccc(S(Nc3ccccc3)(=O)=O)cc2)no1',
                 'c1cc(no1)NC(=O)c1ccc(S(=O)(=O)Nc2ccccc2)cc1'),
    TestMolecule('Nn1c(Cc2c3c(cccc3)ccc2)nnc1SCc1ccccc1',
                 'c1ccc(CSc2nnc([nH]2)Cc2c3c(cccc3)ccc2)cc1'),
    TestMolecule('Cc1[nH]nc(Nc2cc(C)ccc2)c1[N+](=O)[O-]', 'c1ccc(cc1)Nc1n[nH]cc1'),
    TestMolecule('CC1Cn2c(nc3n(C)c(=O)[nH]c(=O)c23)O1', 'O=c1[nH]c2nc3n(c2c([nH]1)=O)CCO3'),
    TestMolecule('c1csc(C(OCC(NC23CC4CC(C2)CC(C3)C4)=O)=O)c1',
                 'c1csc(C(OCC(NC23CC4CC(C2)CC(C3)C4)=O)=O)c1'),
    TestMolecule('c1ccc(S(NC2=NC(=O)C(=Cc3cnccc3)S2)(=O)=O)cc1',
                 'c1ccc(S(NC2=NC(=O)C(=Cc3cnccc3)S2)(=O)=O)cc1'),
    TestMolecule('CCCn1c(N2CCN(C)CC2)nc2n(C)c(=O)[nH]c(=O)c12',
                 'O=c1[nH]c([nH]c2nc([nH]c12)N1CCNCC1)=O'),
    TestMolecule('CCn1c(SCC(Nc2cc(S(N3CCOCC3)(=O)=O)ccc2OC)=O)nnc1-c1ccncc1',
                 'c1cc(S(=O)(=O)N2CCOCC2)cc(NC(=O)CSc2nnc(-c3ccncc3)[nH]2)c1'),
    TestMolecule('C#CCNC(=O)C1=CC(c2ccc(Br)cc2)CC(OCc2ccc(CO)cc2)O1',
                 'c1cccc(c1)C1C=COC(OCc2ccccc2)C1'),
    TestMolecule('CCc1c(SCC(=O)Nc2cc(C)on2)nc2ccc(C)cc2c1', 'O=C(Nc1ccon1)CSc1ccc2c(cccc2)n1'),
    TestMolecule('CCOCCCN(C(C(NC1CCCC1)=O)c1cccc(OC)c1OC)C(c1ccco1)=O',
                 'c1cc(ccc1)C(NC(c1occc1)=O)C(=O)NC1CCCC1'),
    TestMolecule('Cc1ccc(C(=O)NC(=S)NNS(c2ccccc2)(=O)=O)cc1',
                 'c1cccc(c1)C(NC(=S)NNS(=O)(=O)c1ccccc1)=O'),
    TestMolecule('COc1ccc(CC(N)=NOC(=O)c2sccc2)cc1', 'O=C(ON=CCc1ccccc1)c1sccc1'),
    TestMolecule('c1ccc(C(O)=C2C(c3ncccc3)N(CC(OC)OC)C(=O)C2=O)cc1',
                 'c1cc(C=C2C(=O)C(=O)NC2c2ncccc2)ccc1'),
    TestMolecule('COC(=O)CSc1nc(C)cc(Oc2ccccc2)n1', 'c1ccc(Oc2ccncn2)cc1'),
    TestMolecule('COc1ccc(Cn2c(C)ccc2C)cc1', 'c1ccc(cc1)Cn1cccc1'),
    TestMolecule('COc1cccc(N2CCN(C3CC(=O)N(c4ccc(C)c(Cl)c4)C3=O)CC2)c1',
                 'O=C1N(c2ccccc2)C(=O)C(C1)N1CCN(c2ccccc2)CC1'),
    TestMolecule('COc1cccc(OC)c1OCCN(C)C.OC(=O)C(O)=O', 'c1ccccc1'),
    TestMolecule('C1CCC(NC(=O)c2ccc(S(N3CCCC3)(=O)=O)cc2)C1',
                 'C1CCC(NC(=O)c2ccc(S(N3CCCC3)(=O)=O)cc2)C1'),
    TestMolecule('CCCN(C(=O)Cn1ncc2c(=O)oc3c(c12)cccc3)c1cc(C)ccc1',
                 'O=C(Cn1ncc2c(oc3c(cccc3)c12)=O)Nc1ccccc1'),
    TestMolecule('CNC(NC(CSc1nnc(C(F)(F)F)n1C)=O)=O', 'n1nc[nH]c1'),
    TestMolecule('CCOCCCN1C(=O)CC(C(NCCc2ccc(C)cc2)=O)C1', 'O=C1NCC(C1)C(NCCc1ccccc1)=O'),
    TestMolecule('COc1c([N+](=O)[O-])cc(CSc2n[nH]c(C)n2)cc1', 'c1ccc(CSc2nc[nH]n2)cc1'),
    TestMolecule('CN(C)CC(=O)c1ccc(-c2ccccc2)cc1', 'c1cccc(c1)-c1ccccc1'),
    TestMolecule('CC1(O)C(=O)c2c(cccc2)N(c2ccccc2)C1=O', 'O=C1CC(=O)N(c2c1cccc2)c1ccccc1'),
    TestMolecule('CN(S(c1ccccc1)(=O)=O)CC(=O)NCCc1ccccc1', 'c1ccc(CCNC(=O)CNS(=O)(=O)c2ccccc2)cc1'),
    TestMolecule('CCNc1ccccc1C(=O)O', 'c1ccccc1'),
    TestMolecule('CC1(C)C(CSc2nc3ccccc3[nH]2)C1(Cl)Cl', 'c1ccc2c(nc([nH]2)SCC2CC2)c1'),
    TestMolecule('CC(C)c1ccc(OCC(=O)NC(=S)Nc2c3cccc4c3c(cc2)CC4)cc1',
                 'O=C(NC(=S)Nc1c2cccc3c2c(cc1)CC3)COc1ccccc1'),
    TestMolecule('CN(C)c1ccc(NC(CN2CCC(C(c3ccc(F)cc3)=O)CC2)=O)cc1',
                 'c1cccc(c1)NC(CN1CCC(CC1)C(=O)c1ccccc1)=O'),
    TestMolecule('CCCCN(C)C(=O)Cc1c(OC)ccc2cc(Br)ccc21', 'c1c2ccccc2ccc1'),
    TestMolecule('Cc1ccc(NC(CSc2sc(NC(CN3CCOCC3)=O)nn2)=O)cc1',
                 'O=C(Nc1ccccc1)CSc1sc(nn1)NC(=O)CN1CCOCC1'),
    TestMolecule('COCCNC(=S)NNc1cccc(C(=O)O)c1', 'c1ccccc1'),
    TestMolecule('O=C(CNc1ccccc1)NN=Cc1ccc2c(c1)OCCO2', 'O=C(CNc1ccccc1)NN=Cc1ccc2c(c1)OCCO2'),
    TestMolecule('COc1cc2ccccc2cc1C(=O)NCC(c1sccc1)N(C)C', 'O=C(NCCc1sccc1)c1cc2c(cc1)cccc2'),
    TestMolecule('COc1ccc(C(N(C)C)CNC(=O)CCOc2ccccc2)cc1', 'O=C(NCCc1ccccc1)CCOc1ccccc1'),
    TestMolecule('Cl.CCN(CC)CCCN1C(=O)CSC1c1ccc([N+]([O-])=O)cc1', 'O=C1CSC(c2ccccc2)N1'),
    TestMolecule('CCC(Nc1ccc(OC)cc1OC)=C1C(=O)NC(=O)NC1=O', 'c1cc(NC=C2C(=O)NC(=O)NC2=O)ccc1'),
    TestMolecule('c1coc(-c2cc(C(F)(F)F)nc(NCc3ccc(F)cc3)n2)c1', 'c1ccc(CNc2nccc(n2)-c2occc2)cc1'),
    TestMolecule('CCOC(Nc1sc(C)c(C)c1C(OCC)=O)=O', 'c1ccsc1'),
    TestMolecule('O=CN1CCN(C(C(=O)NC2CCCCC2)c2cc3c(cc2[N+]([O-])=O)OCO3)CC1',
                 'O=C(C(N1CCNCC1)c1ccc2c(c1)OCO2)NC1CCCCC1'),
    TestMolecule('COc1cc(C2N(c3ccc(Br)cc3)C(=O)c3n[nH]c(C)c32)ccc1O',
                 'O=C1c2n[nH]cc2C(N1c1ccccc1)c1ccccc1'),
    TestMolecule('c1cc(NC(=O)c2ccccc2[N+]([O-])=O)c(N2CCOCC2)cc1',
                 'O=C(Nc1c(cccc1)N1CCOCC1)c1ccccc1'),
    TestMolecule('N#Cc1cc2c(nc1SCC(=O)N1CCCCC1)CCCCC2', 'O=C(N1CCCCC1)CSc1ccc2c(n1)CCCCC2'),
    TestMolecule('CCN(CC)c1ccc(CN(C(=O)c2cc(OC)c(OC)c(OC)c2)C2CCS(=O)(=O)C2)cc1',
                 'O=S1(=O)CCC(N(Cc2ccccc2)C(=O)c2ccccc2)C1'),
    TestMolecule('COc1cc(NC(=S)N2CCN(Cc3ccccc3)CC2)cc(OC)c1', 'S=C(N1CCN(CC1)Cc1ccccc1)Nc1ccccc1'),
    TestMolecule('CC(=O)C(=CNc1ccc(OCc2ccccc2)cc1)c1ccccc1', 'c1cccc(c1)COc1ccc(NC=Cc2ccccc2)cc1'),
    TestMolecule('CC(C)C(C(NC(C)C(N)=O)=O)NC(C1CCCN1C(OC(C)(C)C)=O)=O', 'C1CCNC1'),
    TestMolecule('CCOc1ccc(N2CC(C(=O)Nc3cccc(S(NC4=NCCC4)(=O)=O)c3)CC2=O)cc1',
                 'c1cccc(c1)N1CC(C(=O)Nc2cccc(S(=O)(=O)NC3=NCCC3)c2)CC1=O'),
    TestMolecule('O=C(NCc1ccccc1Cl)CSc1ccc(-c2cccs2)nn1', 'O=C(NCc1ccccc1)CSc1ccc(nn1)-c1sccc1'),
    TestMolecule('COc1ccc(OC)c(N=c2ssnc2Cl)c1', 'c1cccc(c1)N=c1ssnc1'),
    TestMolecule('CC(=O)C1=C(C)NC(=O)CC1c1c(Cl)cccc1', 'O=C1CC(C=CN1)c1ccccc1'),
    TestMolecule('CCC(=O)N=C(N)Nc1nc(C)c2cc(C)c(C)cc2n1', 'c1cc2c(cc1)ncnc2'),
    TestMolecule('Cc1ccccc1C(OC1OC(=O)C(Cl)=C1Nc1ccc(C(O)=O)cc1)=O',
                 'O=C(OC1OC(C=C1Nc1ccccc1)=O)c1ccccc1'),
    TestMolecule('CCOc1cc(CN2CCC(CO)(Cc3cccc(C(F)(F)F)c3)CC2)ccc1OC',
                 'c1ccc(cc1)CC1CCN(Cc2ccccc2)CC1'),
    TestMolecule('Cc1cc2c([nH]c(=O)c(CCNC(c3cccs3)=O)c2)cc1C',
                 'O=C(NCCc1cc2ccccc2[nH]c1=O)c1cccs1'),
    TestMolecule('Cc1ccc(Nc2cc(=O)[nH]c(=O)[nH]2)cc1C', 'c1cccc(c1)Nc1cc([nH]c([nH]1)=O)=O'),
    TestMolecule('Cc1cc(OCC(=O)NC2CCS(=O)(=O)C2)c2c(oc(=O)c3c2CCC3)c1',
                 'O=C(NC1CCS(=O)(C1)=O)COc1c2c(ccc1)oc(c1c2CCC1)=O'),
    TestMolecule('CCc1sc(NC(CCC(NCCc2ccc(OC)c(OC)c2)=O)=O)nn1',
                 'c1cc(ccc1)CCNC(=O)CCC(=O)Nc1scnn1'),
    TestMolecule('N#CC1=C(SCc2ccccc2)NC(=O)CC1c1ccc(O)cc1', 'O=C1NC(=CC(C1)c1ccccc1)SCc1ccccc1'),
    TestMolecule('O=C(NCCN1CCOCC1)c1csc2c1CCCC2', 'O=C(NCCN1CCOCC1)c1csc2c1CCCC2'),
    TestMolecule('CCCCC(=O)Nc1cc(OC)c(NC(C2CCCCC2)=O)cc1OC', 'O=C(Nc1ccccc1)C1CCCCC1'),
    TestMolecule('Cc1ccc(C(C(C)OC(C2CC(=O)N(C3CCCCC3)C2)=O)=O)cc1',
                 'c1cc(C(=O)COC(C2CC(=O)N(C2)C2CCCCC2)=O)ccc1'),
    TestMolecule('Cc1ccc(S(C(C#N)c2c(N3CCCC3)nc3ccccc3n2)(=O)=O)cc1C',
                 'c1ccc(cc1)S(=O)(=O)Cc1c(nc2ccccc2n1)N1CCCC1'),
    TestMolecule('CC1(C)OC(=O)C(=Cc2[nH]ccc2)C(=O)O1', 'O=C1OCOC(=O)C1=Cc1[nH]ccc1'),
    TestMolecule('Cc1cc(C)cc(Oc2nc3n(cccc3C)c(=O)c2C=C(C#N)C(=O)NC2CCS(=O)(=O)C2)c1',
                 'c1ccc(cc1)Oc1c(c(=O)n2ccccc2n1)C=CC(=O)NC1CCS(=O)(=O)C1'),
    TestMolecule('COc1cc(NC(=O)NCc2c(C)onc2-c2ccccc2)ccc1', 'O=C(NCc1conc1-c1ccccc1)Nc1ccccc1'),
    TestMolecule('c1ccc(C(Oc2cc3c(cc2)C(=O)CO3)=O)cc1', 'c1ccc(C(Oc2cc3c(cc2)C(=O)CO3)=O)cc1'),
    TestMolecule('CCN1C(=O)C2C(c3cccs3)N3C4C(=O)N(CC)C(=O)C4C(c4cccs4)N3C2C1=O',
                 'c1cc(sc1)C1C2C(NC(=O)C2N2N1C1C(=O)NC(=O)C1C2c1cccs1)=O'),
    TestMolecule('Cc1cc(C(N2CCCC(C(c3cc(F)ccc3F)=O)C2)=O)c(C)o1',
                 'O=C(N1CCCC(C(=O)c2ccccc2)C1)c1cocc1'),
    TestMolecule('COc1cc(C=NO)ccc1Oc1c([N+]([O-])=O)cc([N+]([O-])=O)cc1', 'c1cccc(Oc2ccccc2)c1'),
    TestMolecule('Cc1ccc(N(Cc2c(=O)[nH]c3ccc(C)cc3c2)C(c2cccs2)=O)cc1',
                 'O=C(N(c1ccccc1)Cc1c([nH]c2c(cccc2)c1)=O)c1cccs1'),
    TestMolecule('COc1ccc(C(=O)Nn2c(C)nnc2-n2c(C)cc(C)n2)cc1OC', 'O=C(c1ccccc1)Nn1cnnc1-n1nccc1'),
    TestMolecule('Cc1c(NC(=O)c2c(C)c(Cl)c(C)nc2Cl)cccc1', 'O=C(c1cccnc1)Nc1ccccc1'),
    TestMolecule('c1ccc(CNC(CC(C(=O)NCc2ccccc2)c2nc(=O)c3ccccc3[nH]2)=O)cc1',
                 'c1ccc(CNC(CC(C(=O)NCc2ccccc2)c2nc(=O)c3ccccc3[nH]2)=O)cc1'),
    TestMolecule('CNc1n(-c2ccccc2)ncc1[N+](=O)[O-]', 'c1n(ncc1)-c1ccccc1'),
    TestMolecule('CC1SC2(NC1=O)C1CC3CC(C1)CC2C3', 'O=C1CSC2(N1)C1CC3CC(C1)CC2C3'),
    TestMolecule('CCc1ccccc1NC(=S)N(C(C)c1occc1)CCOC', 'S=C(NCc1occc1)Nc1ccccc1'),
    TestMolecule('CCC(C)NC(=O)C1CCCN(S(c2ccc(-n3cnnn3)cc2)(=O)=O)C1',
                 'C1CCN(CC1)S(=O)(=O)c1ccc(cc1)-n1nnnc1'),
    TestMolecule('COc1c2c(ccc1)C1CC(C)(O2)N(Cc2ccccc2)C(=O)N1', 'O=C1NC2CC(Oc3ccccc32)N1Cc1ccccc1'),
    TestMolecule('COc1ccc(C2NC(=O)c3c(cccc3)O2)c(OC)c1OC', 'O=C1NC(Oc2c1cccc2)c1ccccc1'),
    TestMolecule('O=C(NNC=C1C=Nc2ccccc21)c1ccn(Cc2c(Cl)cc(Cl)cc2)n1',
                 'O=C(NNC=C1c2c(cccc2)N=C1)c1nn(cc1)Cc1ccccc1'),
    TestMolecule('c1ccc(NS(c2ccc(OCC(=O)NCc3cnccc3)cc2)(=O)=O)cc1',
                 'c1ccc(NS(c2ccc(OCC(=O)NCc3cnccc3)cc2)(=O)=O)cc1'),
    TestMolecule('COC1=CC(=O)C(=C2NNC(C(F)(F)F)=C2c2cc3ccccc3o2)C=C1',
                 'O=C1C=CC=CC1=C1NNC=C1c1cc2ccccc2o1'),
    TestMolecule('CCOC(=O)c1c(C(COC(C=Cc2ccc(Cl)cc2)=O)=O)c(C)[nH]c1C',
                 'c1ccc(C=CC(OCC(=O)c2cc[nH]c2)=O)cc1'),
    TestMolecule('Cc1nc2ncnn2c(N2CCN(c3nnnn3-c3ccccc3)CC2)c1',
                 'c1nc2ncnn2c(c1)N1CCN(c2nnnn2-c2ccccc2)CC1'),
    TestMolecule('CC(C)Oc1ccc(C(=O)Nc2ccc(NC(c3ccco3)=O)c(Cl)c2)cc1',
                 'O=C(Nc1ccc(cc1)NC(=O)c1ccccc1)c1occc1'),
    TestMolecule('CC(c1ccccc1)NC(C(NCC1OCCC1)=O)=O', 'O=C(NCc1ccccc1)C(=O)NCC1OCCC1'),
    TestMolecule('CCCCOc1ccc(NC(=O)CCSc2nccn2C)cc1', 'O=C(Nc1ccccc1)CCSc1ncc[nH]1'),
    TestMolecule('O=C(OCc1ncccc1)c1oc(COc2c(Cl)cccc2)cc1', 'O=C(OCc1ncccc1)c1ccc(o1)COc1ccccc1'),
    TestMolecule('COc1ccc(C=NNC(=O)OC(C)(C)C)cc1OC', 'c1ccccc1'),
    TestMolecule('CC1CCCCC1NC(COC(c1ccc(S(NCc2ccco2)(=O)=O)cc1)=O)=O',
                 'c1coc(c1)CNS(=O)(=O)c1ccc(cc1)C(=O)OCC(=O)NC1CCCCC1'),
    TestMolecule('Nn1c(SCC(=O)Nc2cccc(F)c2)nnc1C1CCCCC1', 'O=C(CSc1[nH]c(nn1)C1CCCCC1)Nc1ccccc1'),
    TestMolecule('Cc1n[nH]c(NC2CCCCC2)nc1=O', 'O=c1cn[nH]c(n1)NC1CCCCC1'),
    TestMolecule('CCCCCCCCC(=O)NC(C(Cl)(Cl)Cl)NC(=S)N1CCOCC1', 'C1NCCOC1'),
    TestMolecule('CCCc1ccc(Oc2coc3cc(OCC(Nc4c(C)cccc4)=O)ccc3c2=O)cc1',
                 'c1cccc(c1)Oc1c(c2ccc(cc2oc1)OCC(=O)Nc1ccccc1)=O'),
    TestMolecule('Cc1ccc(C(=O)NN=C2CCSC2)cc1[N+]([O-])=O', 'O=C(NN=C1CCSC1)c1ccccc1'),
    TestMolecule('N#CC1=C2SCN(c3ccc(F)cc3)CN2C(=O)CC1c1cc(F)ccc1',
                 'O=C1N2CN(c3ccccc3)CSC2=CC(c2ccccc2)C1'),
    TestMolecule('c1ccc(CN2C(=O)CC(Nc3cc4c(cc3)cccc4)C2=O)cc1',
                 'c1ccc(CN2C(=O)CC(Nc3cc4c(cc3)cccc4)C2=O)cc1'),
    TestMolecule('COc1ccc(NC(C)=O)cc1NC(=O)CN1CCN(CC(=O)Nc2ccc(Cl)cc2)CC1',
                 'O=C(Nc1ccccc1)CN1CCN(CC1)CC(=O)Nc1ccccc1'),
    TestMolecule('Clc1c(Cl)c(C2NC(=O)CCC2[N+]([O-])=O)ccc1', 'O=C1NC(CCC1)c1ccccc1'),
    TestMolecule('CCN(C(=O)CSc1n(-c2ccccc2)c(-c2ccccc2)nn1)CC', 'c1ccc(cc1)-n1cnnc1-c1ccccc1'),
    TestMolecule('CC(=O)CCCCn1cnc2n(C)c(=O)n(C)c(=O)c12', 'O=c1[nH]c(c2c(nc[nH]2)[nH]1)=O'),
    TestMolecule('CC1=NN(c2ccccc2)C(=N)C1=NNc1ccc(Cl)cc1', 'N=C1C(=NNc2ccccc2)C=NN1c1ccccc1'),
    TestMolecule('CCc1ccc(OCC(=O)N(CC)CC)cc1', 'c1ccccc1'),
    TestMolecule('CN(CC(=O)N1CCCCC1)S(c1ccc(Cl)cc1)(=O)=O', 'O=C(CNS(=O)(=O)c1ccccc1)N1CCCCC1'),
    TestMolecule('CSc1ncc(C=C2C(=O)NC(=O)N(c3ccc(C)cc3)C2=O)cn1',
                 'c1ccc(N2C(NC(=O)C(=Cc3cncnc3)C2=O)=O)cc1'),
    TestMolecule('COCCNC(=S)Nc1c(Cc2ccccc2)cccc1', 'c1ccc(Cc2ccccc2)cc1'),
    TestMolecule('COc1cc(C(=O)Nc2nnc(C(C)(C)C)s2)c([N+]([O-])=O)cc1OC', 'O=C(Nc1nncs1)c1ccccc1'),
    TestMolecule('CCOC(=O)c1ccc(NC(=O)c2cc(OC)c(OC(C)C)cc2)cc1', 'O=C(Nc1ccccc1)c1ccccc1'),
    TestMolecule('COc1ccc(C(=O)C=C2Sc3cc4c(cc3N2C)OCO4)cc1', 'O=C(C=C1Sc2cc3c(cc2N1)OCO3)c1ccccc1'),
    TestMolecule('CCCC1=NN(c2sc3c(n2)cccc3)C(=O)C1=CNCCCN(CC)CC', 'C=C1C=NN(C1=O)c1sc2ccccc2n1'),
    TestMolecule('COc1ccc(C(COC(CN2C(=O)NC(C)(C)C2=O)=O)=O)cc1OC',
                 'c1ccc(C(=O)COC(=O)CN2C(=O)CNC2=O)cc1'),
    TestMolecule('O=C(Oc1ccc(Br)cc1)C1CC(=O)N(c2ccc(F)cc2)C1',
                 'O=C(C1CC(N(C1)c1ccccc1)=O)Oc1ccccc1'),
    TestMolecule('O=c1nc(-c2ccccn2)[nH]c(C(F)(F)F)c1Br', 'O=c1cc[nH]c(-c2ncccc2)n1'),
    TestMolecule('CCOC(c1oc2ccccc2c1NC(CN1CCN(C)CC1)=O)=O', 'O=C(CN1CCNCC1)Nc1coc2ccccc21'),
    TestMolecule('CSc1nsc(NN=Cc2ccc3c(c2)OCO3)c1C#N', 'c1cc(sn1)NN=Cc1ccc2OCOc2c1'),
    TestMolecule('CC(C)(C)NC(NC(CSc1nc(C)c(C)c(C)n1)=O)=O', 'c1cncnc1'),
    TestMolecule('Cc1cccnc1CN1CCN(Cc2onc(C(c3ccccc3)c3ccccc3)n2)CC1',
                 'c1cccnc1CN1CCN(CC1)Cc1onc(n1)C(c1ccccc1)c1ccccc1'),
    TestMolecule('COc1ccc(Nc2oc3cc(=O)ccc-3cc2C(=O)Nc2ncccc2)cc1OC',
                 'c1ccc(cc1)Nc1oc2-c(ccc(c2)=O)cc1C(Nc1ncccc1)=O'),
    TestMolecule('c1cc(C)c(OCC(NS(c2ccc(C)cc2)(=O)=O)=O)cc1', 'O=C(COc1ccccc1)NS(=O)(=O)c1ccccc1'),
    TestMolecule('CCOc1ccc(-c2scc(CSc3sc(N)nn3)n2)cc1OC', 'c1cccc(c1)-c1nc(cs1)CSc1scnn1'),
    TestMolecule('c1ccc(C(=O)COC(=O)CN2C(=O)C3C4CC(C3C2=O)C=C4)cc1',
                 'c1ccc(C(=O)COC(=O)CN2C(=O)C3C4CC(C3C2=O)C=C4)cc1'),
    TestMolecule('Cc1occc1C(=O)NC(C)c1ccc2c(c1)OCO2', 'O=C(NCc1ccc2c(c1)OCO2)c1ccoc1'),
    TestMolecule('CCn1c(SCC(=O)Nc2c(Cl)nccc2)nnc1-c1ccccc1',
                 'O=C(Nc1cnccc1)CSc1[nH]c(nn1)-c1ccccc1'),
    TestMolecule('CCC(C)N(C)C1CCN(C(=S)Nc2cc(OC)ccc2)CC1', 'S=C(Nc1ccccc1)N1CCCCC1'),
    TestMolecule('Brc1oc(C(=O)N2CC(=O)Nc3c(cc(Br)cc3)C2c2ccccc2)cc1',
                 'O=C(N1CC(Nc2ccccc2C1c1ccccc1)=O)c1occc1'),
    TestMolecule('CN(C(=O)CCSc1nc(-c2cc3c(cc2)OCO3)cc(C(F)(F)F)n1)Cc1ccccc1',
                 'O=C(NCc1ccccc1)CCSc1nc(ccn1)-c1cc2c(cc1)OCO2'),
    TestMolecule('[Br-].COc1c(OC)c(OC)cc(-c2nc3c[n+](CC(=O)c4ccccc4)ccc3n2C)c1',
                 'O=C(C[n+]1cc2nc([nH]c2cc1)-c1ccccc1)c1ccccc1'),
    TestMolecule('CCOC(CSc1n(-c2c(OC)cccc2)c(CNC(Cc2ccccc2)=O)nn1)=O',
                 'O=C(Cc1ccccc1)NCc1n(cnn1)-c1ccccc1'),
    TestMolecule('CS(N(Cc1ccccc1)c1ccc(C(Nc2c(Sc3ccccc3)cccc2)=O)cc1)(=O)=O',
                 'O=C(c1ccc(NCc2ccccc2)cc1)Nc1c(cccc1)Sc1ccccc1'),
    TestMolecule('Cc1nc(C2N(C(=O)c3cn(C)c4c(c3=O)cccc4)CCc3c4c([nH]c32)cccc4)ccc1',
                 'O=C(c1c[nH]c2c(cccc2)c1=O)N1C(c2ncccc2)c2[nH]c3ccccc3c2CC1'),
    TestMolecule('CCCCc1nc(N2CCOCC2)c(C#N)c2c1CCCC2', 'c1nc(cc2c1CCCC2)N1CCOCC1'),
    TestMolecule('O=C(NN=Cc1cc([N+]([O-])=O)ccc1Cl)c1nccnc1', 'O=C(NN=Cc1ccccc1)c1nccnc1'),
    TestMolecule('COc1ccc(-n2c(SCC(=O)c3ccc4c(c3)OCCO4)nnn2)cc1',
                 'O=C(c1ccc2c(c1)OCCO2)CSc1n(nnn1)-c1ccccc1'),
    TestMolecule('COc1c(C=CC(=O)Nc2cc(S(NC3=NCCCCC3)(=O)=O)ccc2)cccc1',
                 'O=C(Nc1cc(ccc1)S(=O)(=O)NC1=NCCCCC1)C=Cc1ccccc1'),
    TestMolecule('Cc1nn(-c2ccc(F)cc2)c(Cl)c1C=C(CC(=O)O)c1sc2ccccc2n1',
                 'c1cc2sc(nc2cc1)C=Cc1cn(nc1)-c1ccccc1'),
    TestMolecule('COc1c(OC)c(OC)cc(C2N(c3ccccc3)OC3C2C(=O)N(Cc2ccccc2)C3=O)c1',
                 'c1cccc(c1)CN1C(=O)C2C(N(OC2C1=O)c1ccccc1)c1ccccc1'),
    TestMolecule('COCCNC(=S)Nc1cc(OC)c(NC(=O)c2ccco2)cc1OC', 'O=C(Nc1ccccc1)c1occc1'),
    TestMolecule('N#Cc1c(SCC(=O)c2cc3c(oc2=O)cccc3)nc(-c2ccccc2)cc1',
                 'O=C(c1cc2c(cccc2)oc1=O)CSc1cccc(n1)-c1ccccc1'),
    TestMolecule('O=C(N1CCCC1)c1nc2ccccn2c1CN1CCCC(OCc2ccccc2)C1',
                 'O=C(N1CCCC1)c1nc2ccccn2c1CN1CCCC(OCc2ccccc2)C1'),
    TestMolecule('Brc1cccc(OCCSc2ncccn2)c1', 'c1cccc(c1)OCCSc1ncccn1'),
    TestMolecule('CC(C)(C)NC(=O)C12CCC(C)(C1(C)C)c1nc3ccccc3nc12', 'c1cccc2nc3C4CC(CC4)c3nc12'),
    TestMolecule('[I-].CC(C)C1C(OCC(O)C[N+]2(C)CCCCC2)CC(C)CC1', 'C1CC[NH+](CC1)CCCOC1CCCCC1'),
    TestMolecule('Cc1ccccc1NS(=O)(=O)c1ccc(OCC(=O)N2CCCCC2)cc1',
                 'c1cc(ccc1)NS(=O)(=O)c1ccc(cc1)OCC(=O)N1CCCCC1'),
    TestMolecule('Cc1cc(NC(=O)CSc2nc3c(c(=O)n2-c2ccc(Br)cc2)SCC3)no1',
                 'O=C(CSc1nc2c(c(n1-c1ccccc1)=O)SCC2)Nc1ccon1'),
    TestMolecule('Cc1ccccc1C(NC(C(C)C)C(OCC(c1[nH]ccc1)=O)=O)=O',
                 'c1cc([nH]c1)C(COC(CNC(=O)c1ccccc1)=O)=O'),
    TestMolecule('Cc1ccnc(NS(c2ccc(NS(C)(=O)=O)cc2)(=O)=O)n1', 'c1ccc(S(=O)(=O)Nc2ncccn2)cc1'),
    TestMolecule('Cn1c(-c2ccc(Cl)cc2)cnc1NCc1cc2c(cc1[N+]([O-])=O)OCO2.OC(=O)C(O)=O',
                 'c1cc(ccc1)-c1[nH]c(nc1)NCc1cc2c(cc1)OCO2'),
    TestMolecule('CC1Cc2ccccc2N1C(=O)CON=Cc1ccc(OC(F)F)cc1', 'O=C(CON=Cc1ccccc1)N1CCc2c1cccc2'),
    TestMolecule('C=C1C(=O)OC2C(O)C(C)=CC(=O)C=C(C)CC(OC(C(C)=CC)=O)C12',
                 'C=C1C2CCC=CC(C=CCC2OC1=O)=O'),
    TestMolecule('O=C1C2N(CSC2)c2c(cc(C(F)(F)F)cc2)N1Cc1cccc(F)c1',
                 'O=C1C2N(CSC2)c2ccccc2N1Cc1ccccc1'),
    TestMolecule('Cc1ccc(OCC(=O)Nc2c[nH]c(=O)[nH]c2=O)cc1C',
                 'O=C(COc1ccccc1)Nc1c[nH]c([nH]c1=O)=O'),
    TestMolecule('Cn1c(CN2CCOCC2)nc2cc(NC(=O)c3ccccc3Cl)ccc12',
                 'O=C(c1ccccc1)Nc1ccc2[nH]c(nc2c1)CN1CCOCC1'),
    TestMolecule('O=c1oc2ccc(O)cc2c(CN2CCN(CC=Cc3ccccc3)CC2)c1',
                 'O=c1oc2ccccc2c(c1)CN1CCN(CC1)CC=Cc1ccccc1'),
    TestMolecule('Cn1c(Cc2ccccc2)nnc1SCCC(=O)Nc1ccccc1', 'O=C(CCSc1nnc([nH]1)Cc1ccccc1)Nc1ccccc1'),
    TestMolecule('c1cc2nc(CC(=O)c3cc([N+]([O-])=O)ccc3)[nH]c2cc1',
                 'O=C(Cc1nc2ccccc2[nH]1)c1ccccc1'),
    TestMolecule('c1cc2cc(C(=O)N3CCN(c4ccc(N5CCOCC5)nn4)CC3)c(=O)oc2cc1',
                 'c1cc2cc(C(=O)N3CCN(c4ccc(N5CCOCC5)nn4)CC3)c(=O)oc2cc1'),
    TestMolecule('COc1ccccc1-n1c(=S)[nH]nc1CCn1nc(C)c(Br)c1C', 'S=c1[nH]nc(n1-c1ccccc1)CCn1cccn1'),
    TestMolecule('CCC(=O)NC(=S)Nc1ccc(N2CCOCC2)cc1', 'c1cccc(c1)N1CCOCC1'),
    TestMolecule('CCCCCC(=O)N1CCN(CCNC=C2C(=O)CC(c3ccc(OC)c(OC)c3)CC2=O)CC1',
                 'c1ccc(cc1)C1CC(=O)C(C(=O)C1)=CNCCN1CCNCC1'),
    TestMolecule('CN1CCN(C(=O)CN(S(C)(=O)=O)Cc2ccc(Cl)cc2)CC1', 'O=C(CNCc1ccccc1)N1CCNCC1'),
    TestMolecule('COc1cc(OC)cc(C(=O)NCc2cccnc2)c1', 'O=C(NCc1cccnc1)c1ccccc1'),
    TestMolecule('c1cncc(NC(=O)C2CCCN(S(c3cccc4c3nsn4)(=O)=O)C2)c1',
                 'c1cncc(NC(=O)C2CCCN(S(c3cccc4c3nsn4)(=O)=O)C2)c1'),
    TestMolecule('CC(NC1=NN(C(C)=O)C(C)(c2cccs2)S1)=O', 'c1cc(sc1)C1SC=NN1'),
    TestMolecule('CCCC(=O)Nc1ccc(-c2nc3cc(C)c(C)cc3o2)cc1', 'c1cccc(c1)-c1nc2ccccc2o1'),
    TestMolecule('Cc1c(C)n(CC(O)CN2CCOCC2)c2ccccc12.OC(=O)C(O)=O', 'c1cn(c2ccccc12)CCCN1CCOCC1'),
    TestMolecule('Cc1occc1-c1n(CCc2ccccc2)c(SCC(=O)Nc2sccn2)nn1',
                 'O=C(Nc1sccn1)CSc1n(c(nn1)-c1cocc1)CCc1ccccc1'),
    TestMolecule('Cc1oc(-c2cc(F)ccc2)nc1CN1C(CCc2ncccc2)CCCC1',
                 'c1ccc(cc1)-c1nc(co1)CN1C(CCCC1)CCc1ncccc1'),
    TestMolecule('COc1c(OC)c(C(O)=O)c(C=NNC(c2cc(NC(c3ccc(F)cc3)=O)ccc2)=O)cc1',
                 'O=C(Nc1cc(ccc1)C(=O)NN=Cc1ccccc1)c1ccccc1'),
    TestMolecule('CCn1c(Cc2ccccc2)nnc1SCC(=O)Nc1ccc(S(N)(=O)=O)cc1',
                 'O=C(CSc1[nH]c(nn1)Cc1ccccc1)Nc1ccccc1'),
    TestMolecule('CCn1c(COc2nn(-c3ccccc3)c(=O)cc2)nnc1SCc1ccc(OC)cc1',
                 'O=c1ccc(nn1-c1ccccc1)OCc1[nH]c(nn1)SCc1ccccc1'),
    TestMolecule('CC1=NC(=O)C(=C2CC(O)(C(F)(F)F)ON2)C(C)=C1', 'O=C1C(=C2NOCC2)C=CC=N1'),
    TestMolecule('COc1ccc(NC(=S)Nc2ccccc2C(F)(F)F)cc1', 'S=C(Nc1ccccc1)Nc1ccccc1'),
    TestMolecule('CCCc1cc(=O)nc(SCC(=O)c2cc(C)n(CCOC)c2C)[nH]1',
                 'O=C(c1c[nH]cc1)CSc1[nH]ccc(=O)n1'),
    TestMolecule('CC(=O)Nc1ccc2c(c1)C(C)(C)C(C)N2C', 'c1ccc2c(c1)NCC2'),
    TestMolecule('CCN1CCN(C(c2ccc(OCC(Nc3ccc(F)cc3)=O)c(OC)c2)=O)CC1',
                 'c1cc(ccc1)NC(=O)COc1ccc(C(N2CCNCC2)=O)cc1'),
    TestMolecule('CCCCN1C2CCCC1CC(NC(=O)c1ccc(OC)c(OC)c1)C2', 'O=C(NC1CC2NC(CCC2)C1)c1ccccc1'),
    TestMolecule('c1ccc(N(CC(=O)N2CCOCC2)S(c2ccccc2)(=O)=O)cc1',
                 'c1ccc(N(CC(=O)N2CCOCC2)S(c2ccccc2)(=O)=O)cc1'),
    TestMolecule('CCn1c(C)nc2cc(C(=O)NN=Cc3ccc(OC)c(O)c3)ccc12',
                 'O=C(NN=Cc1ccccc1)c1ccc2[nH]cnc2c1'),
    TestMolecule('[Cl-].NC(=O)CN1C=CC(=C[NH+]=O)C=C1', 'C=C1C=CNC=C1'),
    TestMolecule('Cn1cnnc1SC1C(NS(c2ccccc2)(=O)=O)c2c3c(ccc2)cccc31',
                 'O=S(=O)(NC1C(Sc2[nH]cnn2)c2cccc3c2c1ccc3)c1ccccc1'),
    TestMolecule('COc1ccc(Nc2nc(NCc3ccco3)nc(NN=Cc3ccccc3F)n2)cc1',
                 'c1ccc(Nc2nc(nc(n2)NN=Cc2ccccc2)NCc2ccco2)cc1'),
    TestMolecule('CC1=CC(=O)C(=C2C=C(c3ccccc3[N+]([O-])=O)NN2)C=C1',
                 'O=C1C(=C2NNC(=C2)c2ccccc2)C=CC=C1'),
    TestMolecule('COc1ccc(CC2[N+]([O-])(C)CCc3cc(OC)c(O)cc32)cc1O',
                 'c1ccc(cc1)CC1c2c(cccc2)CC[NH2+]1'),
    TestMolecule('Cl.NC(N)=Nc1nc(=O)c2cc(Br)ccc2[nH]1', 'O=c1nc[nH]c2ccccc21'),
    TestMolecule('CC(=O)N1CCC(=NNc2ccc(S(=O)(=O)N3CCOCC3)cc2[N+]([O-])=O)CC1',
                 'c1cc(ccc1NN=C1CCNCC1)S(=O)(=O)N1CCOCC1'),
    TestMolecule('Cc1cc(S(N(Cc2ccc(F)cc2)CC2OCCC2)(=O)=O)ccc1-n1cnnn1',
                 'c1cc(ccc1)CN(CC1OCCC1)S(c1ccc(cc1)-n1cnnn1)(=O)=O'),
    TestMolecule('CC1(C)OCc2c(c3c(sc4c(NCCCO)ncnc43)nc2-c2ccco2)C1',
                 'c1ncnc2c1sc1nc(c3c(c12)CCOC3)-c1ccco1'),
    TestMolecule('COc1ccc(CCNC(=O)CSc2n(-c3ccc(OC)c(OC)c3)nnn2)cc1OC',
                 'O=C(CSc1n(-c2ccccc2)nnn1)NCCc1ccccc1'),
    TestMolecule('CC(C)(CC(O)=O)CC(NCc1c(Cl)cccc1Sc1ccc(Cl)cc1)=O', 'c1ccc(Sc2ccccc2)cc1'),
    TestMolecule('COc1ccc(-c2cc(CCCC(=O)NCCc3cc(OC)ccc3OC)no2)cc1',
                 'O=C(NCCc1ccccc1)CCCc1noc(c1)-c1ccccc1'),
    TestMolecule('Cc1ccc(-c2ncns2)cc1', 'c1ccc(cc1)-c1sncn1'),
    TestMolecule('C(O)CCn1c(=O)c2c(nc1C=Cc1ccc([N+]([O-])=O)o1)cccc2',
                 'O=c1[nH]c(C=Cc2ccco2)nc2c1cccc2'),
    TestMolecule('COC(CC(O)CC(O)C(C)OCc1ccccc1)OC', 'c1ccccc1'),
    TestMolecule('Cl.CCCC(N1CCN(C(=O)c2occc2)CC1)c1n(C(C)(C)C)nnn1',
                 'O=C(N1CCN(Cc2nnn[nH]2)CC1)c1ccco1'),
    TestMolecule('O=C(NC(CO)c1ccccc1)c1occc1', 'O=C(NCc1ccccc1)c1occc1'),
    TestMolecule('O=C(Nc1ccc(N2CCOCC2)cc1)c1c(Cl)cc(F)c(F)c1', 'O=C(Nc1ccc(N2CCOCC2)cc1)c1ccccc1'),
    TestMolecule('CCc1sc(N2C(=O)c3ccc(Oc4ccc([N+]([O-])=O)cc4)cc3C2=O)nn1',
                 'O=C1N(C(=O)c2cc(Oc3ccccc3)ccc21)c1scnn1'),
    TestMolecule('CC(C)Cc1ccc(C(C)C(=O)O)cc1', 'c1ccccc1'),
    TestMolecule('Cl.N=c1sccn1CC(=O)Nc1cc(S(N2CCCC2)(=O)=O)ccc1Cl',
                 'N=c1n(CC(=O)Nc2cccc(S(=O)(N3CCCC3)=O)c2)ccs1'),
    TestMolecule('c1ccc(-c2ccc(C(=O)OC3CC4OC(=O)CC4C3CO)cc2)cc1',
                 'c1ccc(cc1)-c1ccc(C(=O)OC2CC3CC(=O)OC3C2)cc1'),
    TestMolecule('CN(CCC#N)CC(=O)Nc1ccc(S(N)(=O)=O)cc1', 'c1ccccc1'),
    TestMolecule('Cc1nc(-c2ccc([N+]([O-])=O)cc2)sc1C(=O)O', 'c1cc(-c2sccn2)ccc1'),
    TestMolecule('c1coc(C(=O)N2CCN(C(Cn3nnc(-c4ccc(NC(c5ccc(F)cc5)=O)cc4)n3)=O)CC2)c1',
                 'O=C(N1CCN(C(=O)Cn2nc(nn2)-c2ccc(NC(=O)c3ccccc3)cc2)CC1)c1ccco1'),
    TestMolecule('Cc1onc(-c2c(Cl)cccc2Cl)c1C(N)=S', 'c1ccc(cc1)-c1nocc1'),
    TestMolecule('CCOC(=O)c1cnc2ccccc2c1NCCO', 'c1cnc2ccccc2c1'),
    TestMolecule('Cc1ccc(C)c(NC(=O)Cn2nnc(-c3ccc(N4CCOCC4)cc3)n2)c1',
                 'O=C(Cn1nnc(n1)-c1ccc(cc1)N1CCOCC1)Nc1ccccc1'),
    TestMolecule('CC(C)(C)c1cc(C(=O)NNc2ccc(OC(F)(F)F)cc2)n(Cc2ccccc2)n1',
                 'O=C(NNc1ccccc1)c1ccnn1Cc1ccccc1'),
    TestMolecule('CCCCCOC(=O)C1=C(C)N=C2N(NN=N2)C1c1ccc(OC)c(OC)c1OC',
                 'c1cccc(c1)C1N2NN=NC2=NC=C1'),
    TestMolecule('Cc1cc2cc(CNC(=O)C3CC3)ccc2n1C', 'O=C(NCc1ccc2c(cc[nH]2)c1)C1CC1'),
    TestMolecule('Cc1ccccc1C(NC(CC(C)C)C(Nc1cc(S(N(C)C)(=O)=O)ccc1)=O)=O',
                 'c1ccc(cc1)NC(CNC(=O)c1ccccc1)=O'),
    TestMolecule('COCCCNC(=S)N1CCC(NC(=O)c2ccco2)CC1', 'O=C(NC1CCNCC1)c1ccco1'),
    TestMolecule('Cn1c(C=Cc2oc([N+]([O-])=O)cc2)nc2ccccc2c1=O', 'O=c1[nH]c(C=Cc2occc2)nc2ccccc12'),
    TestMolecule('c1cc2nc(SCc3cc(=O)n4ccsc4n3)n(CCCO)c(=O)c2cc1',
                 'c1ccc2nc(SCc3cc(=O)n4ccsc4n3)[nH]c(=O)c2c1'),
    TestMolecule('c1ccc2c(c1)cccc2NC(=O)CC1SC(NCC2OCCC2)=NC1=O',
                 'c1ccc2c(c1)cccc2NC(=O)CC1SC(NCC2OCCC2)=NC1=O'),
  ]


if __name__ == '__main__':  # pragma: no cover
  unittest.main()



#
#  Copyright (C) 2002-2008  greg Landrum and Rational Discovery LLC
#
""" unit tests for the model and descriptor packager """
import os
import pickle
import unittest
from io import BytesIO
from xml.dom import minidom
from xml.etree import ElementTree as ET

from rdkit import Chem, RDConfig
from rdkit import RDRandom as random
from rdkit.Chem import Descriptors
from rdkit.ML.Composite import Composite
from rdkit.ML.Data import DataUtils
from rdkit.ML.Descriptors.MoleculeDescriptors import \
    MolecularDescriptorCalculator
from rdkit.ML.ModelPackage import Packager, PackageUtils
from rdkit.ML.ModelPackage.Packager import ModelPackage


def feq(a, b, tol=1e-4):
  return abs(a - b) <= tol


class TestCase(unittest.TestCase):

  def setUp(self):
    self.dataDir = os.path.join(RDConfig.RDCodeDir, 'ML/ModelPackage/test_data')
    self.testD = [
      # NOTE: the confidences here can be twitchy due to changes in descriptors:
      ('Fc1ccc(NC(=O)c2cccnc2Oc3cccc(c3)C(F)(F)F)c(F)c1', 0, 0.8),
      # (r'CN/1(=C\C=C(/C=C1)\C\2=C\C=N(C)(Cl)\C=C2)Cl',0,0.70),
      (r'NS(=O)(=O)c1cc(ccc1Cl)C2(O)NC(=O)c3ccccc32', 1, 0.70),
    ]

  def _loadPackage(self):
    with open(os.path.join(self.dataDir, 'Jan9_build3_pkg.pkl'), 'r') as pkgTF:
      buf = pkgTF.read().replace('\r\n', '\n').encode('utf-8')
      pkgTF.close()
    io = BytesIO(buf)
    pkg = pickle.load(io)
    return pkg

  def _verify(self, pkg, testD):
    for smi, pred, conf in testD:
      m = Chem.MolFromSmiles(smi)
      self.assertTrue(m is not None, 'SMILES: %s failed\n' % (smi))
      p, c = pkg.Classify(m)
      assert p == pred, 'bad prediction (%d) for smiles %s' % (p, smi)
      assert feq(c, conf), 'bad confidence (%f) for smiles %s' % (c, smi)

  def _verify2(self, pkg, testD):
    for smi, pred, conf in testD:
      m = Chem.MolFromSmiles(smi)
      self.assertTrue(m is not None, 'SMILES: %s failed\n' % (smi))
      p, c = pkg.Classify(m)
      assert p == pred, 'bad prediction (%d) for smiles %s' % (p, smi)
      assert feq(c, conf), 'bad confidence (%f) for smiles %s' % (c, smi)
      p, c = pkg.Classify(m)
      assert p == pred, 'bad prediction (%d) for smiles %s' % (p, smi)
      assert feq(c, conf), 'bad confidence (%f) for smiles %s' % (c, smi)

  def testBuild(self):
    # """ tests building and screening a packager """
    with open(os.path.join(self.dataDir, 'Jan9_build3_calc.dsc'), 'r') as calcTF:
      buf = calcTF.read().replace('\r\n', '\n').encode('utf-8')
      calcTF.close()
    calc = pickle.load(BytesIO(buf))
    with open(os.path.join(self.dataDir, 'Jan9_build3_model.pkl'), 'rb') as modelF:
      model = pickle.load(modelF)
    pkg = Packager.ModelPackage(descCalc=calc, model=model)
    self._verify(pkg, self.testD)

  def testLoad(self):
    # """ tests loading and screening a packager """
    pkg = self._loadPackage()
    self._verify(pkg, self.testD)

  def testLoad2(self):
    # """ tests loading and screening a packager 2 """
    pkg = self._loadPackage()
    self._verify2(pkg, self.testD)

  def testPerm1(self):
    # """ tests the descriptor remapping stuff in a packager """
    pkg = self._loadPackage()
    calc = pkg.GetCalculator()
    names = calc.GetDescriptorNames()
    ref = {}
    DataUtils.InitRandomNumbers((23, 42))
    for smi, _, _ in self.testD:
      for desc in names:
        fn = getattr(Descriptors, desc, lambda x: 777)
        m = Chem.MolFromSmiles(smi)
        ref[desc] = fn(m)

      for _ in range(5):
        perm = list(names)
        random.shuffle(perm, random=random.random)

        m = Chem.MolFromSmiles(smi)
        for desc in perm:
          fn = getattr(Descriptors, desc, lambda x: 777)
          val = fn(m)
          assert feq(val, ref[desc],
                     1e-4), '%s: %s(%s): %f!=%f' % (str(perm), smi, desc, val, ref[desc])

  def testPerm2(self):
    # """ tests the descriptor remapping stuff in a packager """
    pkg = self._loadPackage()
    calc = pkg.GetCalculator()
    names = calc.GetDescriptorNames()
    DataUtils.InitRandomNumbers((23, 42))
    perm = list(names)
    random.shuffle(perm, random=random.random)
    calc.simpleList = perm
    calc.descriptorNames = perm
    pkg.Init()
    self._verify(pkg, self.testD)

  def test_ModelPackage(self):
    pkg = self._loadPackage()

    self.assertTrue(isinstance(pkg.GetCalculator(), MolecularDescriptorCalculator))
    pkg.SetCalculator('calculator')
    self.assertEqual(pkg.GetCalculator(), 'calculator')

    self.assertTrue(isinstance(pkg.GetModel(), Composite.Composite))
    pkg.SetModel('model')
    self.assertEqual(pkg.GetModel(), 'model')

    self.assertEqual(pkg.GetDataset(), None)
    pkg.SetDataset('dataset')
    self.assertEqual(pkg.GetDataset(), 'dataset')

    self.assertEqual(pkg.GetNotes(), 'General purpose model built from PhysProp data')
    pkg.SetNotes('notes')
    self.assertEqual(pkg.GetNotes(), 'notes')

    # Here seems to be a difference between Python 2 and 3. The next assert works in Python 3,
    # but fails in Python 2
    # self.assertFalse(hasattr(pkg, '_supplementalData'))
    self.assertEqual(pkg.GetSupplementalData(), [])
    self.assertTrue(hasattr(pkg, '_supplementalData'))

    delattr(pkg, '_supplementalData')
    pkg.AddSupplementalData('supp1')
    self.assertTrue(hasattr(pkg, '_supplementalData'))
    self.assertEqual(pkg.GetSupplementalData(), ['supp1'])
    pkg.AddSupplementalData('supp2')
    self.assertEqual(pkg.GetSupplementalData(), ['supp1', 'supp2'])

    pkg = ModelPackage()
    self.assertFalse(pkg._initialized)
    pkg.Init()
    self.assertFalse(pkg._initialized)

  def test_PackageUtils(self):
    pkg = self._loadPackage()
    xml = PackageUtils.PackageToXml(
      pkg, dataPerformance=[
        ('label', ['accuracy', 'avgCorrect', 'avgIncorrect']),
      ], recommendedThreshold=0.2, classDescriptions=[('a', 'texta'), ('b', 'textb')],
      modelType='model type', modelOrganism='model organism')
    s = prettyXML(xml.getroot())
    self.assertIn('<RDModelInfo>', s)


def prettyXML(xml):
  s = ET.tostring(xml, encoding='utf-8')
  tree = minidom.parseString(s)
  return tree.toprettyxml(indent=' ')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import io
import os
import pickle
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import rdPartialCharges


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test0HalgrenSet(self):
    smiSup = Chem.SmilesMolSupplier(
      os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'PartialCharges', 'Wrap', 'test_data',
                   'halgren.smi'), delimiter='\t')

    # parse the original file
    with open(
        os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'PartialCharges', 'Wrap', 'test_data',
                     'halgren_out.txt'), 'r') as infil:
      lines = infil.readlines()

    tab = Chem.GetPeriodicTable()

    olst = []
    for mol in smiSup:
      rdPartialCharges.ComputeGasteigerCharges(mol)
      tstr = "Molecule: "
      tstr += mol.GetProp("_Name")
      olst.append(tstr)
      for i in range(mol.GetNumAtoms()):
        at = mol.GetAtomWithIdx(i)
        en = tab.GetElementSymbol(at.GetAtomicNum())
        chg = float(at.GetProp("_GasteigerCharge"))
        tstr = "%i %s %6.4f" % (i, en, chg)
        olst.append(tstr)

    i = 0
    for line in lines:
      self.assertTrue(line.strip() == olst[i])
      i += 1

  def test1PPDataset(self):
    fileN = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'PartialCharges', 'Wrap',
                         'test_data', 'PP_descrs_regress.2.csv')
    infil = open(fileN, 'r')
    lines = infil.readlines()
    infil.close()

    infile = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'PartialCharges', 'Wrap',
                          'test_data', 'PP_combi_charges.pkl')
    with open(infile, 'r') as cchtFile:
      buf = cchtFile.read().replace('\r\n', '\n').encode('utf-8')
      cchtFile.close()
    with io.BytesIO(buf) as cchFile:
      combiCharges = pickle.load(cchFile)

    for lin in lines:
      if (lin[0] == '#'):
        continue
      tlst = lin.strip().split(',')
      smi = tlst[0]
      rdmol = Chem.MolFromSmiles(smi)
      rdPartialCharges.ComputeGasteigerCharges(rdmol)

      nat = rdmol.GetNumAtoms()
      failed = False
      for ai in range(nat):
        rdch = float(rdmol.GetAtomWithIdx(ai).GetProp('_GasteigerCharge'))
        if not feq(rdch, combiCharges[smi][ai], 1.e-2):
          failed = True
          print(smi, ai, rdch, combiCharges[smi][ai])
      if failed:
        rdmol.Debug()
      self.assertFalse(failed)

  def test2Params(self):
    """ tests handling of Issue187 """
    m1 = Chem.MolFromSmiles('C(=O)[O-]')
    rdPartialCharges.ComputeGasteigerCharges(m1)

    m2 = Chem.MolFromSmiles('C(=O)[O-].[Na+]')
    rdPartialCharges.ComputeGasteigerCharges(m2)

    for i in range(m1.GetNumAtoms()):
      c1 = float(m1.GetAtomWithIdx(i).GetProp('_GasteigerCharge'))
      c2 = float(m2.GetAtomWithIdx(i).GetProp('_GasteigerCharge'))
      self.assertTrue(feq(c1, c2, 1e-4))

  def test3Params(self):
    """ tests handling of Issue187 """
    m2 = Chem.MolFromSmiles('C(=O)[O-].[Na+]')
    with self.assertRaisesRegex(Exception, ""):
      rdPartialCharges.ComputeGasteigerCharges(m2, 12, 1)

  def testGithubIssue20(self):
    """ tests handling of Github issue 20 """
    m1 = Chem.MolFromSmiles('CB(O)O')
    rdPartialCharges.ComputeGasteigerCharges(m1)
    chgs = [-0.030, 0.448, -0.427, -0.427]
    for i in range(m1.GetNumAtoms()):
      c1 = float(m1.GetAtomWithIdx(i).GetProp('_GasteigerCharge'))
      self.assertAlmostEqual(c1, chgs[i], 3)

  def testGithubIssue577(self):
    """ tests handling of Github issue 577 """
    m1 = Chem.MolFromSmiles('CCO')
    from locale import LC_NUMERIC, setlocale
    try:
      setlocale(LC_NUMERIC, "de_DE")
    except Exception:
      # can't set the required locale, might as well just return
      return
    try:
      rdPartialCharges.ComputeGasteigerCharges(m1)
      for at in m1.GetAtoms():
        float(at.GetProp('_GasteigerCharge'))
    finally:
      setlocale(LC_NUMERIC, "C")
    rdPartialCharges.ComputeGasteigerCharges(m1)
    for at in m1.GetAtoms():
      float(at.GetProp('_GasteigerCharge'))

  def testGithub2480(self):
    with self.assertRaisesRegex(Exception, "^Python argument types"):
      rdPartialCharges.ComputeGasteigerCharges(None)


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2007  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import doctest
import gzip
import os
import pickle
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem.AtomPairs import Pairs, Sheridan, Torsions, Utils


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(Pairs, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(Sheridan, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(Torsions, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(Utils, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def setUp(self):
    self.testDataPath = os.path.join(RDConfig.RDCodeDir, 'Chem', 'AtomPairs', 'test_data')
    inF = gzip.open(os.path.join(self.testDataPath, 'mols1000.pkl.gz'), 'rb')
    self.mols = pickle.load(inF, encoding='bytes')

  def testPairsRegression(self):
    inF = gzip.open(os.path.join(self.testDataPath, 'mols1000.aps.pkl.gz'), 'rb')
    atomPairs = pickle.load(inF, encoding='bytes')
    for i, m in enumerate(self.mols):
      ap = Pairs.GetAtomPairFingerprint(m)
      if ap != atomPairs[i]:  # pragma: nocover
        debugFingerprint(m, ap, atomPairs[i])
      self.assertEqual(ap, atomPairs[i])
      self.assertNotEqual(ap, atomPairs[i - 1])

  def testTorsionsRegression(self):
    inF = gzip.open(os.path.join(self.testDataPath, 'mols1000.tts.pkl.gz'), 'rb')
    torsions = pickle.load(inF, encoding='bytes')
    for i, m in enumerate(self.mols):
      tt = Torsions.GetTopologicalTorsionFingerprintAsIntVect(m)
      if tt != torsions[i]:  # pragma: nocover
        debugFingerprint(m, tt, torsions[i])
      self.assertEqual(tt, torsions[i])
      self.assertNotEqual(tt, torsions[i - 1])

  def testGetTopologicalTorsionFingerprintAsIds(self):
    mol = Chem.MolFromSmiles('C1CCCCN1')
    tt = Torsions.GetTopologicalTorsionFingerprint(mol)
    self.assertEqual(tt.GetNonzeroElements(), {4437590049: 2, 8732557345: 2, 4445978657: 2})
    tt = Torsions.GetTopologicalTorsionFingerprintAsIds(mol)
    self.assertEqual(sorted(tt),
                     [4437590049, 4437590049, 4445978657, 4445978657, 8732557345, 8732557345])
    tt = Torsions.GetTopologicalTorsionFingerprintAsIntVect(mol)
    self.assertEqual(tt.GetNonzeroElements(), {4437590049: 2, 8732557345: 2, 4445978657: 2})

  def testGithub334(self):
    m1 = Chem.MolFromSmiles('N#C')
    self.assertEqual(Utils.NumPiElectrons(m1.GetAtomWithIdx(0)), 2)
    self.assertEqual(Utils.NumPiElectrons(m1.GetAtomWithIdx(1)), 2)

    m1 = Chem.MolFromSmiles('N#[CH]')
    self.assertEqual(Utils.NumPiElectrons(m1.GetAtomWithIdx(0)), 2)
    self.assertEqual(Utils.NumPiElectrons(m1.GetAtomWithIdx(1)), 2)


def debugFingerprint(mol, fpCalc, fpExpected):  # pragma: nocover
  print(Chem.MolToSmiles(mol))
  pd = fpCalc.GetNonzeroElements()
  rd = fpExpected.GetNonzeroElements()
  for k, v in pd.items():
    if k in rd:
      if rd[k] != v:
        print('>>>1', k, v, rd[k])
    else:
      print('>>>2', k, v)
  for k, v in rd.items():
    if k in pd:
      if pd[k] != v:
        print('>>>3', k, v, pd[k])
    else:
      print('>>>4', k, v)


if __name__ == '__main__':
  unittest.main()



import unittest

from rdkit import Chem
from rdkit.Chem import rdDeprotect as rd


class TestCase(unittest.TestCase):

  def testDeprotect(self):
    smiles = "N(C(=O)OC(C)(C)C)Cc1ccccc1NC(=O)OC(C)(C)C"
    m = Chem.MolFromSmiles(smiles)
    m2 = rd.Deprotect(m)
    self.assertEqual(Chem.MolToSmiles(m2), Chem.CanonSmiles("NCc1ccccc1N"))
    self.assertEqual(list(m2.GetPropsAsDict()["DEPROTECTIONS"]), ["Boc", "Boc"])
    self.assertEqual(m2.GetPropsAsDict()["DEPROTECTION_COUNT"], 2)

  def testDeprotectInPlace(self):
    smiles = "N(C(=O)OC(C)(C)C)Cc1ccccc1NC(=O)OC(C)(C)C"
    m = Chem.MolFromSmiles(smiles)
    self.assertTrue(rd.DeprotectInPlace(m))
    self.assertEqual(Chem.MolToSmiles(m), Chem.CanonSmiles("NCc1ccccc1N"))
    self.assertEqual(list(m.GetPropsAsDict()["DEPROTECTIONS"]), ["Boc", "Boc"])
    self.assertEqual(m.GetPropsAsDict()["DEPROTECTION_COUNT"], 2)

  def testDeprotectVector(self):
    reaction_class = "amine"
    reaction_smarts = "[C;R0][C;R0]([C;R0])([O;R0][C;R0](=[O;R0])[NX3;H0,H1:1])C>>[N:1]"
    abbreviation = "Boc"
    full_name = "tert-butyloxycarbonyl"
    data = rd.DeprotectData(reaction_class, reaction_smarts, abbreviation, full_name)

    smiles = "N(C(=O)OC(C)(C)C)Cc1ccccc1NC(=O)OC(C)(C)C"
    m = Chem.MolFromSmiles(smiles)
    m2 = rd.Deprotect(m, [])
    self.assertEqual(Chem.MolToSmiles(m), Chem.MolToSmiles(m2))
    self.assertEqual(m2.GetPropsAsDict()["DEPROTECTION_COUNT"], 0)

    m2 = rd.Deprotect(m, [data])
    self.assertEqual(Chem.MolToSmiles(m2), Chem.CanonSmiles("NCc1ccccc1N"))
    self.assertEqual(list(m2.GetPropsAsDict()["DEPROTECTIONS"]), ["Boc", "Boc"])
    self.assertEqual(m2.GetPropsAsDict()["DEPROTECTION_COUNT"], 2)

  def test_examples(self):
    count = 0
    for data in rd.GetDeprotections():
      if data.example:
        start, end = data.example.split(">>")
        m = Chem.MolFromSmiles(start)
        m2 = rd.Deprotect(m, [data])
        print("Testing", data.full_name)
        self.assertEqual(Chem.MolToSmiles(m2), Chem.CanonSmiles(end))
        count += 1
    assert count

  def test_examples_inplace(self):
    count = 0
    for data in rd.GetDeprotections():
      if data.example:
        start, end = data.example.split(">>")
        m = Chem.MolFromSmiles(start)
        rd.DeprotectInPlace(m, [data])
        print("Testing", data.full_name)
        self.assertEqual(Chem.MolToSmiles(m), Chem.CanonSmiles(end))
        count += 1
    assert count


if __name__ == "__main__":
  unittest.main()



#
#  Copyright (C) 2001  greg Landrum
#
""" unit testing code for the descriptor COM server

"""

import unittest

import numpy as np

from rdkit import RDConfig

try:
  from win32com.client import Dispatch
except ImportError:
  Dispatch = None


@unittest.skipIf(Dispatch is None, 'Windows test')
class TestCase(unittest.TestCase):

  def setUp(self):
    print('\n%s: ' % self.shortDescription(), end='')

  def testConnectToCOMServer(self):
    # " testing connection "
    Dispatch('RD.DescCalc')

  def testLoadCalculator(self):
    # " testing load "
    c = Dispatch('RD.DescCalc')
    c.LoadCalculator(RDConfig.RDCodeDir + '/ml/descriptors/test_data/ferro.dsc')

  def testNames(self):
    # " testing GetDescriptorNames "
    c = Dispatch('RD.DescCalc')
    c.LoadCalculator(RDConfig.RDCodeDir + '/ml/descriptors/test_data/ferro.dsc')
    names = c.GetDescriptorNames()
    expectedNames = ('MAX_DED', 'has3d', 'has4d', 'has5d', 'elconc', 'atvol')
    assert names == expectedNames, 'GetDescriptorNames failed (%s != %s)' % (repr(names),
                                                                             repr(expectedNames))

  def testCalc(self):
    # " testing descriptor calculation "
    argV = ['CrPt3', 'fcc', 'AuCu3', 58.09549962, 1, 4, 0.228898, 8.876, 1]
    nameV = [
      'Compound', 'Structure', 'Structure_Type', 'Volume', 'Z', 'Atoms_per_Formula_Unit',
      'Hardness', 'RawDOS_Ef', 'IsFerromagnetic'
    ]
    c = Dispatch('RD.DescCalc')
    c.LoadCalculator(RDConfig.RDCodeDir + '/ml/descriptors/test_data/ferro.dsc')
    descVect = np.array(c.CalcDescriptors(argV, nameV))
    expected = np.array((3.67481803894, 1, 0, 1, 0.619669341609, 14.523874905))
    diffV = abs(descVect - expected)
    assert max(diffV) < 0.0001, 'bad descriptors: %s, %s' % (str(expected), str(descVect))


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2000  greg Landrum
#
""" unit testing code for cross validation """

import os
import pickle
import unittest
from io import BytesIO, StringIO

from rdkit import RDConfig, RDRandom
from rdkit.ML.DecTree import CrossValidate, randomtest
from rdkit.TestRunner import redirect_stdout


class XValTestCase(unittest.TestCase):

  def setUp(self):
    self.origTreeName = RDConfig.RDCodeDir + '/ML/DecTree/test_data/XValTree.pkl'
    self.randomSeed = 23
    self.randomArraySeed = (23, 42)

  def testRun(self):
    # " test that the CrossValidationDriver runs "
    examples, attrs, nPossibleVals = randomtest.GenRandomExamples(nExamples=200)
    f = StringIO()
    with redirect_stdout(f):
      tree, frac = CrossValidate.CrossValidationDriver(examples, attrs, nPossibleVals, silent=False)
    self.assertGreater(frac, 0)
    self.assertEqual('Var: 1', tree.GetName())
    self.assertIn('Validation error', f.getvalue())

    CrossValidate.CrossValidationDriver(examples, attrs, nPossibleVals, lessGreedy=True,
                                        calcTotalError=True, silent=True)

  def testResults(self):
    # " test the results of CrossValidation "
    RDRandom.seed(self.randomSeed)
    examples, attrs, nPossibleVals = randomtest.GenRandomExamples(nExamples=200,
                                                                  seed=self.randomArraySeed)
    tree, frac = CrossValidate.CrossValidationDriver(examples, attrs, nPossibleVals, silent=1)
    self.assertGreater(frac, 0)

    with open(self.origTreeName, 'r') as inTFile:
      buf = inTFile.read().replace('\r\n', '\n').encode('utf-8')
      inTFile.close()
    inFile = BytesIO(buf)
    oTree = pickle.load(inFile)

    assert oTree == tree, 'Random CrossValidation test failed'

  def testReplacementSelection(self):
    # " use selection with replacement "
    RDRandom.seed(self.randomSeed)
    examples, attrs, nPossibleVals = randomtest.GenRandomExamples(nExamples=200,
                                                                  seed=self.randomArraySeed)
    tree, frac = CrossValidate.CrossValidationDriver(examples, attrs, nPossibleVals, silent=1,
                                                     replacementSelection=1)
    self.assertTrue(tree)
    self.assertAlmostEqual(frac, 0.01666, 4)

  def test_TestRun(self):
    try:
      f = StringIO()
      with redirect_stdout(f):
        CrossValidate.TestRun()
      self.assertTrue(os.path.isfile('save.pkl'))
      s = f.getvalue()
      self.assertIn('t1 == t2 True', s)
    finally:
      if os.path.isfile('save.pkl'):
        os.remove('save.pkl')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import unittest

from rdkit import Chem


class TestCase(unittest.TestCase):

  def testLabelAtomsList(self):
    mol = Chem.MolFromSmiles("C[C@H](Cl)CC[C@H](Cl)C")
    self.assertIsNotNone(mol)

    atom1 = mol.GetAtomWithIdx(1)
    atom5 = mol.GetAtomWithIdx(5)

    self.assertTrue(atom1.HasProp("_CIPCode"))
    self.assertTrue(atom5.HasProp("_CIPCode"))

    atom1.ClearProp("_CIPCode")
    atom5.ClearProp("_CIPCode")

    Chem.rdCIPLabeler.AssignCIPLabels(mol, [1], None)

    self.assertEqual(atom1.GetProp("_CIPCode"), "S")
    self.assertFalse(atom5.HasProp("_CIPCode"))

  def testLabelAtomsSet(self):
    mol = Chem.MolFromSmiles("C[C@H](Cl)CC[C@H](Cl)C")
    self.assertIsNotNone(mol)

    atom1 = mol.GetAtomWithIdx(1)
    atom5 = mol.GetAtomWithIdx(5)

    self.assertTrue(atom1.HasProp("_CIPCode"))
    self.assertTrue(atom5.HasProp("_CIPCode"))

    atom1.ClearProp("_CIPCode")
    atom5.ClearProp("_CIPCode")

    Chem.rdCIPLabeler.AssignCIPLabels(mol, {1}, None)

    self.assertEqual(atom1.GetProp("_CIPCode"), "S")
    self.assertFalse(atom5.HasProp("_CIPCode"))

  def testLabelBondsList(self):
    mol = Chem.MolFromSmiles(r"C\C=C\C=C/C")
    self.assertIsNotNone(mol)

    bond1 = mol.GetBondWithIdx(1)
    bond3 = mol.GetBondWithIdx(3)

    self.assertEqual(bond1.GetBondType(), Chem.BondType.DOUBLE)
    self.assertEqual(bond3.GetBondType(), Chem.BondType.DOUBLE)

    self.assertFalse(bond1.HasProp("_CIPCode"))
    self.assertFalse(bond3.HasProp("_CIPCode"))

    Chem.rdCIPLabeler.AssignCIPLabels(mol, None, [3])

    self.assertFalse(bond1.HasProp("_CIPCode"))
    self.assertEqual(bond3.GetProp("_CIPCode"), "Z")


if __name__ == '__main__':
  unittest.main()



#
#  Copyright (C) 2007-2017 Greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" General descriptor testing code

"""

import doctest
import io
import os.path
import pickle
import unittest

import numpy as np

from rdkit import Chem, RDConfig
from rdkit.Chem import AllChem, Descriptors, Lipinski, rdMolDescriptors, Descriptors3D


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(Descriptors, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def testGithub1287(self):
    smis = ('CCC', )
    for smi in smis:
      m = Chem.MolFromSmiles(smi)
      self.assertTrue(m)
      for nm, fn in Descriptors._descList:
        try:
          _ = fn(m)
        except Exception:
          import traceback
          traceback.print_exc()
          raise AssertionError('SMILES: %s; Descriptor: %s' % (smi, nm))

  def testBadAtomHandling(self):
    smis = ('CC[Pu]', 'CC[*]')
    for smi in smis:
      m = Chem.MolFromSmiles(smi)
      self.assertTrue(m)
      for nm, fn in Descriptors._descList:
        try:
          v = fn(m)
        except RuntimeError:
          # 3D descriptors fail since the mol has no conformers
          pass
        except Exception:
          import traceback
          traceback.print_exc()
          raise AssertionError('SMILES: %s; Descriptor: %s' % (smi, nm))

  def testMolFormula(self):
    for (smiles, expected) in (
      ("[NH4+]", "H4N+"),
      ("c1ccccc1", "C6H6"),
      ("C1CCCCC1", "C6H12"),
      ("c1ccccc1O", "C6H6O"),
      ("C1CCCCC1O", "C6H12O"),
      ("C1CCCCC1=O", "C6H10O"),
      ("N[Na]", "H2NNa"),
      ("[C-][C-]", "C2-2"),
      ("[H]", "H"),
      ("[H-1]", "H-"),
      ("[H-1]", "H-"),
      ("[CH2]", "CH2"),
      ("[He-2]", "He-2"),
      ("[U+3]", "U+3"),
    ):
      mol = Chem.MolFromSmiles(smiles)
      actual = AllChem.CalcMolFormula(mol)
      self.assertEqual(actual, expected)

  def testMQNDetails(self):
    refFile = os.path.join(os.path.dirname(__file__), 'test_data', 'MQNs_regress.pkl')
    refFile2 = os.path.join(os.path.dirname(__file__), 'test_data', 'MQNs_non_strict_regress.pkl')
    # figure out which definition we are currently using
    m = Chem.MolFromSmiles("CC(C)(C)c1cc(O)c(cc1O)C(C)(C)C")
    if Lipinski.NumRotatableBonds(m) == 2:
      refFile = refFile2

    with open(refFile, 'rb') as intf:
      refData = pickle.load(intf)
    fn = os.path.join(os.path.dirname(__file__), 'test_data', 'aromat_regress.txt')
    ms = [x for x in Chem.SmilesMolSupplier(fn, delimiter='\t')]
    for i, m in enumerate(ms):
      mqns = rdMolDescriptors.MQNs_(m)
      if mqns != refData[i][1]:
        indices = [(j, x, y) for j, x, y in zip(range(len(mqns)), mqns, refData[i][1]) if x != y]
        print(i, Chem.MolToSmiles(m), indices)
      self.assertEqual(mqns, refData[i][1])

  def testMQN(self):
    m = Chem.MolFromSmiles("CC(C)(C)c1cc(O)c(cc1O)C(C)(C)C")
    if Lipinski.NumRotatableBonds(m) == 2:
      tgt = [
        42917, 274, 870, 621, 135, 1582, 29, 3147, 5463, 6999, 470, 62588, 19055, 4424, 309, 24061,
        17820, 1, 9303, 24146, 16076, 5560, 4262, 646, 746, 13725, 5430, 2629, 362, 24211, 15939,
        292, 41, 20, 1852, 5642, 31, 9, 1, 2, 3060, 1750
      ]
    else:
      tgt = [
        42917, 274, 870, 621, 135, 1582, 29, 3147, 5463, 6999, 470, 62588, 19055, 4424, 309, 24061,
        17820, 1, 8314, 24146, 16076, 5560, 4262, 646, 746, 13725, 5430, 2629, 362, 24211, 15939,
        292, 41, 20, 1852, 5642, 31, 9, 1, 2, 3060, 1750
      ]
      tgt = [
        42917, 274, 870, 621, 135, 1582, 29, 3147, 5463, 6999, 470, 62588, 19055, 4424, 309, 24059,
        17822, 1, 8314, 24146, 16076, 5560, 4262, 646, 746, 13725, 5430, 2629, 362, 24211, 15939,
        292, 41, 20, 1852, 5642, 31, 9, 1, 2, 3060, 1750
      ]
    fn = os.path.join(os.path.dirname(__file__), 'test_data', 'aromat_regress.txt')
    ms = [x for x in Chem.SmilesMolSupplier(fn, delimiter='\t')]
    vs = np.zeros((42, ), np.int32)

    for m in ms:
      vs += rdMolDescriptors.MQNs_(m)
    self.assertEqual(list(vs), tgt)

  def test_FpDensityMorgan(self):
    self.assertEqual(Descriptors.FpDensityMorgan1.version, '1.0.0')
    self.assertEqual(Descriptors.FpDensityMorgan2.version, '1.0.0')
    self.assertEqual(Descriptors.FpDensityMorgan3.version, '1.0.0')

    m = Chem.MolFromSmiles('C')
    self.assertAlmostEqual(Descriptors.FpDensityMorgan2(m), 1)
    m = Chem.MolFromSmiles('CC')
    self.assertAlmostEqual(Descriptors.FpDensityMorgan2(m), 1)
    m = Chem.MolFromSmiles('CCC')
    self.assertAlmostEqual(Descriptors.FpDensityMorgan2(m), 4.0 / 3)
    m = Chem.MolFromSmiles('C' * 10)
    self.assertAlmostEqual(Descriptors.FpDensityMorgan2(m), 8.0 / 10)
    m = Chem.MolFromSmiles('C' * 100)
    self.assertAlmostEqual(Descriptors.FpDensityMorgan2(m), 8.0 / 100)

    m = Chem.MolFromSmiles('CCCc1ccccc1')
    fpd1 = Descriptors.FpDensityMorgan1(m)
    fpd2 = Descriptors.FpDensityMorgan2(m)
    fpd3 = Descriptors.FpDensityMorgan3(m)
    self.assertAlmostEqual(fpd1, 10.0 / 9)
    self.assertLess(fpd1, fpd2)
    self.assertLess(fpd2, fpd3)

  @unittest.skipIf(not hasattr(rdMolDescriptors, 'BCUT2D'), "BCUT descriptor not available")
  def testVectorDescriptors(self):
    m = Chem.MolFromSmiles('CCCc1ccccc1')
    results = rdMolDescriptors.BCUT2D(m)
    names = [
      "BCUT2D_%s" % s
      for s in ('MWHI', "MWLOW", "CHGHI", "CHGLO", "LOGPHI", "LOGPLOW", "MRHI", "MRLOW")
    ]
    for i, n in enumerate(names):
      f = getattr(Descriptors, n)
      self.assertEqual(results[i], f(m))

    results = rdMolDescriptors.CalcAUTOCORR2D(m)
    names = ["AUTOCORR2D_%s" % str(i + 1) for i in range(192)]
    for i, n in enumerate(names):
      f = getattr(Descriptors, n)
      self.assertEqual(results[i], f(m))

  @unittest.skipIf(not hasattr(rdMolDescriptors, 'BCUT2D')
                   or not hasattr(rdMolDescriptors, 'CalcAUTOCORR2D'),
                   "BCUT or AUTOCORR descriptors not available")
  def testVectorDescriptorsInDescList(self):
    # First try only bcuts should exist
    descriptors = set([n for n, _ in Descriptors.descList])
    names = set([
      "BCUT2D_%s" % i
      for i in ('MWHI', "MWLOW", "CHGHI", "CHGLO", "LOGPHI", "LOGPLOW", "MRHI", "MRLOW")
    ])
    self.assertEqual(descriptors.intersection(names), names)

    Descriptors.setupAUTOCorrDescriptors()
    descriptors2 = set([n for n, _ in Descriptors.descList])
    self.assertEqual(descriptors2.intersection(descriptors), descriptors)

    names = set(["AUTOCORR2D_%s" % str(i + 1) for i in range(192)])
    self.assertEqual(descriptors2.intersection(names), names)

  def test_issue_4567(self):
    """
      Github issue #4567:
      Requesting "density" fingerprint Hydrogen molecule fails with exception
      """
    mol = AllChem.MolFromSmiles('[HH]')
    self.assertEqual(Descriptors.FpDensityMorgan1(mol), 0)

  def testGetMolDescriptors(self):
    mol = Chem.MolFromSmiles('CCCO')
    descs = Descriptors.CalcMolDescriptors(mol)
    self.assertTrue('MolLogP' in descs)
    self.assertEqual(descs['NumHDonors'], 1)

  def testGet3DMolDescriptors(self):
    mol = Chem.MolFromSmiles('CCCO')

    # check ValueError raised when no 3D coordinates supplied
    with self.assertRaises(ValueError):
      Descriptors3D.CalcMolDescriptors3D(mol)

    # test function returns expected outputs
    AllChem.EmbedMolecule(mol, randomSeed=0xf00d)
    descs = Descriptors3D.CalcMolDescriptors3D(mol)
    self.assertTrue('InertialShapeFactor' in descs)
    self.assertEqual(descs['PMI1'], 20.954531335493417)

if __name__ == '__main__':
  unittest.main()



"""
This test currently fails. The database is not available.

"""

import os
import pickle
import unittest

from rdkit import RDConfig
from rdkit.Dbase.DbConnection import DbConnect
from rdkit.ML import InfoTheory

RDConfig.usePgSQL = 0


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


def getFingerprints(conn):
  data = conn.GetData(table='signatures', fields='mol_name,fingerprint')
  fpMap = {}
  for dat in data:
    pkl = str(dat[1])
    sbv = pickle.loads(pkl)
    fpMap[dat[0]] = sbv
  return fpMap


def getNameAct(conn):
  data = conn.GetData(table='raw_data', fields='mol_name,activity_class')
  nameAct = {}
  for dat in data:
    nameAct[dat[0]] = dat[1]

  return nameAct


def ReadCombiInfo(fileName):
  infil = open(fileName, 'r')
  lines = infil.readlines()
  infil.close()
  infos = []
  for lin in lines:
    tlst = lin.strip().split()
    info = float(tlst[1])
    infos.append(info)
  return infos


_testDatabase = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'test_data',
                             'FEW_CDK2.GDB')


@unittest.skipIf(not os.path.isfile(_testDatabase), 'Test database FEW_CDK2.GDB missing')
class TestCase(unittest.TestCase):

  def test0Ranker(self):
    nbits = 5000
    conn = DbConnect(_testDatabase)
    fps = getFingerprints(conn)
    nameAct = getNameAct(conn)
    sl = len(list(fps.values())[0])
    rnkr = InfoTheory.InfoBitRanker(sl, 2, InfoTheory.InfoType.ENTROPY)

    print("Collecting Votes ....")
    for key in nameAct.keys():
      if nameAct[key] == 100:
        rnkr.AccumulateVotes(fps[key], 0)
      if nameAct[key] == 0:
        rnkr.AccumulateVotes(fps[key], 1)

    # now do the ranking
    print("ranking bits ....")
    topN = rnkr.GetTopN(nbits)

    # get the combichem ranked list from a file
    cfile = os.path.join('test_data', 'combiRank.out')
    combiInfo = ReadCombiInfo(cfile)
    # now check if the infocontents are the same as the combichem stuff
    print("Comparing bit info contents ....")
    for i in range(900):
      assert feq(topN[i, 1], combiInfo[i])

    ofile = os.path.join('test_data', 'rdTopBits.txt')
    rnkr.WriteTopBitsToFile(ofile)

  @unittest.skipIf(not os.path.isfile(_testDatabase), 'Test database FEW_CDK2.GDB missing')
  def test1BiasRanker(self):
    nbits = 5000
    conn = DbConnect(_testDatabase)
    fps = getFingerprints(conn)
    nameAct = getNameAct(conn)
    sl = len(list(fps.values())[0])
    rnkr = InfoTheory.InfoBitRanker(sl, 2, InfoTheory.InfoType.BIASENTROPY)
    rnkr.SetBiasList([0])
    print("Collecting Votes ....")
    for key in nameAct.keys():
      if nameAct[key] == 100:
        rnkr.AccumulateVotes(fps[key], 0)
      if nameAct[key] == 0:
        rnkr.AccumulateVotes(fps[key], 1)

    # now do the ranking
    print("ranking bits ....")
    topN = rnkr.GetTopN(nbits)

    # get the combichem ranked list from a file
    cfile = os.path.join('test_data', 'combiRank.out')
    combiInfo = ReadCombiInfo(cfile)
    # now check if the infocontents are the same as the combichem stuff
    print("Comparing bit info contents ....")
    for i in range(nbits):
      assert feq(topN[i, 1], combiInfo[i])

  def test2ChiSquare(self):
    nbits = 5000
    conn = DbConnect(_testDatabase)
    fps = getFingerprints(conn)
    nameAct = getNameAct(conn)
    print(fps.values())
    sl = len(list(fps.values())[0])
    rnkr = InfoTheory.InfoBitRanker(sl, 2, InfoTheory.InfoType.BIASCHISQUARE)
    rnkr.SetBiasList([0])
    print("Collecting Votes ....")
    for key in nameAct.keys():
      if nameAct[key] == 100:
        rnkr.AccumulateVotes(fps[key], 0)
      if nameAct[key] == 0:
        rnkr.AccumulateVotes(fps[key], 1)

    # now do the ranking
    print("ranking bits ....")
    topN = rnkr.GetTopN(nbits)

    # get the combichem ranked list from a file
    cfile = os.path.join('test_data', 'combiRankChi.out')
    combiInfo = ReadCombiInfo(cfile)
    # now check if the infocontents are the same as the combichem stuff
    print("Comparing bit info contents ....")
    for i in range(nbits):
      assert feq(topN[i, 1], combiInfo[i])
    # rnkr.WriteTopBitsToFile("chiBitsBias.txt")


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
# Created by Gareth Jones on 5/30/2020.
#
# Copyright 2020-2022 Schrodinger, Inc and other RDKit contributors
#  @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
""" A rough test of Tautomer Queries 

Rough in that only basic functionality is evaluated.
"""

import os
import pickle
from unittest import TestCase, main
import unittest
from rdkit import Chem, DataStructs
from rdkit.Chem import rdTautomerQuery
from rdkit import rdBase


class TautomerQueryTestCase(TestCase):

  def test_basic(self):
    mol = Chem.MolFromSmiles("O=C1CCCCC1")
    tautomer_query = rdTautomerQuery.TautomerQuery(mol)
    self.assertEqual(2, len(tautomer_query.GetTautomers()))
    modified_bonds = tautomer_query.GetModifiedBonds()
    self.assertEqual(3, len(modified_bonds))
    modified_atoms = tautomer_query.GetModifiedAtoms()
    self.assertEqual(3, len(modified_atoms))

    target = Chem.MolFromSmiles("OC1=CCCC(CC)C1")
    self.assertTrue(target.HasSubstructMatch(tautomer_query.GetTemplateMolecule()))

    match = tautomer_query.IsSubstructOf(target)
    self.assertTrue(match)
    self.assertTrue(target.HasSubstructMatch(tautomer_query.GetTemplateMolecule()))
    #self.assertTrue(target.HasSubstructMatch(tautomer_query.GetTautomers()[1]))
    match = tautomer_query.GetSubstructMatch(target)
    self.assertEqual(7, len(match))

    matches = tautomer_query.GetSubstructMatches(target)
    self.assertEqual(1, len(matches))

    tautomer_matches = tautomer_query.GetSubstructMatchesWithTautomers(target)
    self.assertEqual(1, len(tautomer_matches))
    self.assertEqual(len(tautomer_matches[0][0]), 7)
    matching_tautomer = tautomer_matches[0][1]
    self.assertTrue(target.HasSubstructMatch(matching_tautomer))

    tautomer_fingerprint = tautomer_query.PatternFingerprintTemplate()
    target_fingerprint = rdTautomerQuery.PatternFingerprintTautomerTarget(target)
    matching_fingerprints = DataStructs.AllProbeBitsMatch(tautomer_fingerprint, target_fingerprint)
    self.assertTrue(matching_fingerprints)

    file = os.environ['RDBASE'] + '/Code/GraphMol/MolStandardize/test_data/tautomerTransforms.in'
    tautomer_query2 = rdTautomerQuery.TautomerQuery(mol, file)
    self.assertEqual(2, len(tautomer_query.GetTautomers()))

  def test_parameter_searches(self):
    mol = Chem.MolFromSmiles("O=C1CCCCC1")
    tautomer_query = rdTautomerQuery.TautomerQuery(mol)
    target = Chem.MolFromSmiles("O=C1CCCC(CC)C1")
    params = Chem.SubstructMatchParameters()
    params.uniquify = False
    tautomer_matches = tautomer_query.GetSubstructMatchesWithTautomers(target, params)
    self.assertEqual(2, len(tautomer_matches))
    matches = tautomer_query.GetSubstructMatches(target, params)
    self.assertEqual(2, len(matches))
    match = tautomer_query.GetSubstructMatch(target, params)
    self.assertEqual(7, len(match))
    is_match = tautomer_query.IsSubstructOf(target, params)
    self.assertTrue(is_match)

  def test_types(self):
    mol = Chem.MolFromSmiles("O=C1CCCCC1")
    tautomer_query = rdTautomerQuery.TautomerQuery(mol)
    target = Chem.MolFromSmiles("OC1=CCCC(CC)C1")
    try:
      self.assertTrue(target.HasSubstructMatch(tautomer_query.GetTautomers()[1]))
    except Exception:
      self.fail("Boost type error")

  def test_simple(self):
    mol = Chem.MolFromSmiles("CC=O")
    tautomer_query = rdTautomerQuery.TautomerQuery(mol)
    target = Chem.MolFromSmiles("OC(C)=O")
    tautomers = tautomer_query.GetTautomers()
    self.assertTrue(target.HasSubstructMatch(mol))
    self.assertTrue(tautomer_query.IsSubstructOf(target))

  @unittest.skipIf(not rdBase._serializationEnabled, "not built with serialization support")
  def test_serialization(self):
    mol = Chem.MolFromSmiles("O=C1CCCCC1")
    base_tautomer_query = rdTautomerQuery.TautomerQuery(mol)
    for tautomer_query in (pickle.loads(pickle.dumps(base_tautomer_query)),
                           rdTautomerQuery.TautomerQuery(base_tautomer_query.ToBinary())):
      self.assertEqual(2, len(tautomer_query.GetTautomers()))
      modified_bonds = tautomer_query.GetModifiedBonds()
      self.assertEqual(3, len(modified_bonds))
      modified_atoms = tautomer_query.GetModifiedAtoms()
      self.assertEqual(3, len(modified_atoms))

      target = Chem.MolFromSmiles("OC1=CCCC(CC)C1")
      self.assertTrue(target.HasSubstructMatch(tautomer_query.GetTemplateMolecule()))


if __name__ == "__main__":
  main()



import csv
import doctest
import os
import unittest
from random import Random

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.EnumerateHeterocycles import (EnumerateHeterocycles,
                                              GetHeterocycleReactions,
                                              GetHeterocycleReactionSmarts)


def has_radical(mol):
  for atom in mol.GetAtoms():
    if atom.GetNumRadicalElectrons():
      return True
  return False


def has_aromatic(mol):
  for atom in mol.GetAtoms():
    if atom.GetIsAromatic():
      return True
  return False


def check_smiles(smiles):
  mol = Chem.MolFromSmiles(smiles)
  assert mol is not None, smiles
  assert not has_radical(mol)
  assert len(Chem.rdmolops.GetMolFrags(mol)) == 1, smiles
  return mol


def assert_valid_change(orig_can_smi, smiles):
  orig_mol = Chem.MolFromSmiles(orig_can_smi)
  assert orig_can_smi != smiles, "%s != %s" % (orig_can_smi, smiles)
  msg = "%s produced %s" % (orig_can_smi, smiles)
  try:
    mol = check_smiles(smiles)
  except Exception:
    print(msg)
    raise

  assert has_aromatic(mol), 'lost aromaticity when ' + msg

  orig_charge = Chem.rdmolops.GetFormalCharge(orig_mol)
  new_charge = Chem.rdmolops.GetFormalCharge(mol)

  if orig_charge != new_charge:
    assert orig_charge - 1 == new_charge, msg


def get_unique_products(rxn, mol):
  uniq_smiles = set()
  for newmol in rxn.RunReactants((mol, )):
    isosmiles = Chem.MolToSmiles(newmol[0], isomericSmiles=True)
    if isosmiles in uniq_smiles:
      continue
    uniq_smiles.add(isosmiles)
    newmol = Chem.MolFromSmiles(isosmiles)
    assert newmol is not None, '%s produced %s when applying' % (Chem.MolToSmiles(
      mol, isomericSmiles=True), isosmiles)
    yield Chem.MolToSmiles(newmol, isomericSmiles=True)


class TestCase(unittest.TestCase):

  def test_smarts_match_examples(self):
    for row in GetHeterocycleReactionSmarts():
      smarts = row.SMARTS
      if not smarts:
        continue

      substruct = Chem.MolFromSmarts(smarts)
      for smiles in row.EXAMPLE.split(','):
        assert smiles
        mol = Chem.MolFromSmiles(smiles)
        assert mol.HasSubstructMatch(substruct), "%s not in %s" % (smarts, smiles)

      for smiles in row.NEGATIVE_EXAMPLE.split(','):
        if not smiles:
          continue

        mol = Chem.MolFromSmiles(smiles)
        assert not mol.HasSubstructMatch(substruct), "%s should not be in %s" % (smarts, smiles)

  def test_at_least_something_matches_every_negative_example(self):
    negative_examples = []
    substructs = []
    for row in GetHeterocycleReactionSmarts():
      for smiles in row.NEGATIVE_EXAMPLE.split(','):
        if not smiles:
          continue
        mol = Chem.MolFromSmiles(smiles)
        assert mol is not None
        negative_examples.append(mol)

      smarts = row.SMARTS
      if not smarts:
        continue

      substruct = Chem.MolFromSmarts(smarts)
      assert substruct is not None
      substructs.append(substruct)

    for mol in negative_examples:
      something_hit = False
      for substruct in substructs:
        if mol.HasSubstructMatch(substruct):
          something_hit = True
          break
      assert something_hit, ('nothing matched %s' % Chem.MolToSmiles(mol, isomericSmiles=True))

  def test_reactions_modify_examples(self):
    for row in GetHeterocycleReactionSmarts():
      smarts = row.SMARTS
      if not smarts:
        continue

      for product in row.CONVERT_TO.split(','):
        reaction = smarts + '>>' + product
        rxn = AllChem.ReactionFromSmarts(reaction)

        for smiles in row.EXAMPLE.split(','):
          orig_can_smi = Chem.CanonSmiles(smiles)
          assert smiles
          mol = Chem.MolFromSmiles(smiles)
          for newmol in rxn.RunReactants((mol, )):
            newmol = newmol[0]
            isosmi = Chem.MolToSmiles(newmol, isomericSmiles=True)
            assert_valid_change(orig_can_smi, isosmi)

  def test_apply_every_rule_to_every_fragment(self):
    fieldnames = ['SMILES', 'MUTATED', 'REACTION', 'DESCRIPTION']
    writer = csv.DictWriter(open('hetero_atom_mutations.csv', 'w'), fieldnames)
    writer.writeheader()

    notchanged = csv.DictWriter(open('not_changed.csv', 'w'), ['SMILES', 'TITLE'])
    notchanged.writeheader()

    fragment_library = os.path.join(os.path.dirname(__file__), 'test_data', 'fragments.csv')
    frag_reader = csv.DictReader(open(fragment_library))
    for row in frag_reader:
      smiles = row['SMILES']
      rdkit_mol = Chem.MolFromSmiles(smiles)
      orig_can_smi = Chem.MolToSmiles(rdkit_mol, isomericSmiles=True)

      changed = False

      for src, rxn in zip(GetHeterocycleReactionSmarts(), GetHeterocycleReactions()):
        for smiles in get_unique_products(rxn, rdkit_mol):
          assert_valid_change(orig_can_smi, smiles)
          row = {
            'SMILES': orig_can_smi,
            'MUTATED': smiles,
            'REACTION': src.SMARTS + '>>' + src.CONVERT_TO,
            'DESCRIPTION': src.DESCRIPTION,
          }
          writer.writerow(row)
          changed = True

      # record aromatic fragments that no rule changes (possible problems?)
      if not changed and has_aromatic(rdkit_mol):
        row = {'SMILES': orig_can_smi, 'TITLE': orig_can_smi}
        notchanged.writerow(row)


# There are only 2 possible 6 member ring 3 heteroatom systems, 3 carbons in each (operating under the assumption we never seen 4+!)
# c1ccnnn1 => c1ccnnn1, c1cnnnc1, c1nnncc1
# c1cnncn1 => c1cnncn1, c1nncnc1, c1ncnnc1

# Or we can enumerate all carbons for every 2 heteroatom system, 10 total (safer as it if 4 heteroatoms are passed in, it won't add more)
# c1cccnn1 => c1cccnn1, c1ccnnc1, c1cnncc1, c1nnccc1
# c1ccncn1 => c1ccncn1, c1cncnc1, c1ncncc1, c1ncccn1
# c1cnccn1 => c1cnccn1, c1nccnc1 (other 2 are symmetry)

# $(a1cccnn1),$(a1ccnnc1),$(a1cnncc1),$(a1nnccc1),$(a1ccncn1),$(a1cncnc1),$(a1ncncc1),$(a1ncccn1),$(a1cnccn1),$(a1nccnc1) =>
# only one row needs to allow carbons
# [c;h1;D2;r6;$(a1ccc[c,n,o][c,n,o]1),$(a1cc[c,n,o][c,n,o]c1),$(a1c[c,n,o][c,n,o]cc1),$(a1[c,n,o][c,n,o]ccc1),$(a1cc[n,o]c[n,o]1),$(a1c[n,o]c[n,o]c1),$(a1[n,o]c[n,o]cc1),$(a1[n,o]ccc[n,o]1),$(a1c[n,o]cc[n,o]1),$(a1[n,o]cc[n,o]c1):1]

  def get_six_member_ring_carbon_to_nitrogen_reaction(self):
    expected_description = 'aromatic carbon in 6 membered ring'
    rxns = [
      r for r in GetHeterocycleReactionSmarts() if r.DESCRIPTION.startswith(expected_description)
    ]
    assert len(rxns) == 1, "expecting only one of these rules for now"
    return rxns[0]

  def test_six_member_ring_carbon_to_nitrogen_should_hit_all_other_carbons(self):
    rxn = self.get_six_member_ring_carbon_to_nitrogen_reaction()

    test_inputs = [
      #SMILES, num expected matches
      ('c1ccccc1', 6),
      ('n1ccccc1', 5),
      ('n1ncccc1', 4),
      ('n1cnccc1', 4),
      ('n1ccncc1', 4),
      ('c1c(*)[nH]c(=O)[nH]c1=O', 1),
      ('*c1c[nH]c(=O)[nH]c1=O', 1),
    ]

    smarts_mol = Chem.MolFromSmarts(rxn.SMARTS)

    for smiles, expected_num_matches in test_inputs:
      rdkit_mol = Chem.MolFromSmiles(smiles)
      num_rdkit_matches = len(rdkit_mol.GetSubstructMatches(smarts_mol))

      assert num_rdkit_matches == expected_num_matches, "expected %u matches in %s, got %u" % (
        expected_num_matches, smiles, num_rdkit_matches)

  def test_fuzz_atom_mutations(self):
    fragment_library = os.path.join(os.path.dirname(__file__), 'test_data', 'fragments.csv')
    base, ext = os.path.splitext(os.path.basename(fragment_library))

    rand = Random(0xDEADBEEF)
    uniq_fragments = set()
    fragments = []

    frag_reader = csv.DictReader(open(fragment_library))
    for row in frag_reader:
      smiles = row['SMILES']
      rdkit_mol = Chem.MolFromSmiles(smiles)
      if not has_aromatic(rdkit_mol):
        continue
      orig_can_smi = Chem.MolToSmiles(rdkit_mol, isomericSmiles=True)
      assert orig_can_smi not in uniq_fragments
      uniq_fragments.add(orig_can_smi)
      fragments.append(orig_can_smi)

    print(len(fragments), "fragments with aromaticity")

    fieldnames = ['SMILES', 'MUTATED', 'REACTION', 'DESCRIPTION']
    writer = csv.DictWriter(open(base + 'hetero_atom_mutations_fuzzing.csv', 'w'), fieldnames)
    writer.writeheader()
    notchanged = csv.DictWriter(open(base + 'not_changed_during_fuzzing.csv', 'w'),
                                ['SMILES', 'TITLE'])
    notchanged.writeheader()
    uniq_notchanged = set()

    num_trials = 1000
    # to test the full range of possible fragments
    #num_trials = 1000000

    total_generated = 0
    for i in range(num_trials):
      if i and i % 1000 == 0:
        print(i)
      if not fragments:
        print("Converged! No more fragments left!")
        break
      idx = rand.randint(0, len(fragments) - 1)
      orig_can_smi = fragments.pop(idx)
      rdkit_mol = Chem.MolFromSmiles(orig_can_smi)

      changed = False

      for src, rxn in zip(GetHeterocycleReactionSmarts(), GetHeterocycleReactions()):
        for smiles in get_unique_products(rxn, rdkit_mol):
          total_generated += 1
          changed = True
          assert_valid_change(orig_can_smi, smiles)
          if smiles in uniq_fragments:
            continue
          uniq_fragments.add(smiles)
          fragments.append(smiles)

          row = {
            'SMILES': orig_can_smi,
            'MUTATED': smiles,
            'REACTION': src.SMARTS + '>>' + src.CONVERT_TO,
            'DESCRIPTION': src.DESCRIPTION,
          }
          writer.writerow(row)

      # record aromatic fragments that no rule changes (possible problems?)
      if not changed and orig_can_smi not in uniq_notchanged:
        uniq_notchanged.add(orig_can_smi)
        row = {'SMILES': orig_can_smi, 'TITLE': orig_can_smi}
        notchanged.writerow(row)
    print(total_generated, "generated of which", len(uniq_fragments),
          "are unique fragments generated after", num_trials, "trials")

  def test_reaction_on_mol_with_attachments(self):
    src = self.get_six_member_ring_carbon_to_nitrogen_reaction()
    reaction = src.SMARTS + '>>' + src.CONVERT_TO
    rxn = AllChem.ReactionFromSmarts(reaction)

    rdkit_mol = Chem.MolFromSmiles('c1([*:2])ccccc1([*:3])')
    products = set(get_unique_products(rxn, rdkit_mol))
    expected = set([
      'c1cnc([*:2])c([*:3])c1', 'c1cc([*:3])c([*:2])cn1', 'c1cnc([*:3])c([*:2])c1',
      'c1cc([*:2])c([*:3])cn1'
    ])
    assert products == expected, '%r != %r' % (products, expected)

    # two less of these because of symmetry
    rdkit_mol = Chem.MolFromSmiles('c1([*:2])ccc([*:3])cc1')
    assert set(get_unique_products(rxn, rdkit_mol)) == set(
      ['c1cc([*:2])ncc1[*:3]', 'c1cc([*:3])ncc1[*:2]'])

    # test what happens when nothing matches
    rdkit_mol = Chem.MolFromSmiles('c1([*:2])cc[nH]c1([*:3])')
    assert set(get_unique_products(rxn, rdkit_mol)) == set()

  def test_enumerate_heterocycles(self):
    rand = Random(0xBAD1DEA)

    fragment_library = os.path.join(os.path.dirname(__file__), 'test_data', 'fragments.csv')
    frag_reader = csv.DictReader(open(fragment_library))
    for row in frag_reader:
      smiles = row['SMILES']
      mol = Chem.MolFromSmiles(smiles)
      reference_set = set(Chem.MolToSmiles(m) for m in EnumerateHeterocycles(mol))

      if not has_aromatic(Chem.MolFromSmiles(smiles)):
        assert len(reference_set) == 0
        continue

      assert len(reference_set) > 0

      for nbr_smiles in rand.sample(list(reference_set), 4):
        nbr = Chem.MolFromSmiles(nbr_smiles)
        comparison_set = set(Chem.MolToSmiles(m) for m in EnumerateHeterocycles(nbr))
        assert reference_set == comparison_set

if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2003  Rational Discovery LLC
#   All Rights Reserved
""" unit testing code for knn models """
import doctest
import os.path
import unittest

from rdkit import RDConfig, RDRandom
from rdkit.ML.Data import DataUtils
from rdkit.ML.KNN import (CrossValidate, DistFunctions, KNNModel,
                          KNNRegressionModel)


def feq(a, b, tol=1e-4):
  return abs(a - b) < tol


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(DistFunctions, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def setUp(self):
    RDRandom.seed(25)

  def test1Neighbors(self):
    fName = os.path.join(RDConfig.RDCodeDir, 'ML', 'KNN', 'test_data', 'random_pts.csv')
    data = DataUtils.TextFileToData(fName)
    examples = data.GetNamedData()
    npvals = data.GetNPossibleVals()
    nvars = data.GetNVars()
    attrs = list(range(1, nvars + 1))
    numNeigh = 11
    metric = DistFunctions.EuclideanDist
    mdl = KNNModel.KNNModel(numNeigh, attrs, metric)
    pt = examples.pop(0)
    tgt = [(metric(pt, ex, attrs), ex) for ex in examples]
    tgt.sort()
    mdl.SetTrainingExamples(examples)
    neighbors = mdl.GetNeighbors(pt)
    for i in range(numNeigh):
      assert feq(-tgt[i][0], neighbors[i][0])
      assert tgt[i][1][0] == neighbors[i][1][0]

  def test2XValClass(self):
    fName = os.path.join(RDConfig.RDCodeDir, 'ML', 'KNN', 'test_data', 'random_pts.csv')
    data = DataUtils.TextFileToData(fName)
    examples = data.GetNamedData()
    npvals = data.GetNPossibleVals()
    nvars = data.GetNVars()
    attrs = list(range(1, nvars + 1))
    numNeigh = 11
    mod, err = CrossValidate.CrossValidationDriver(examples, attrs, npvals, numNeigh, silent=1)
    self.assertAlmostEqual(err, 0.01075, 4)

    neighborList = []
    res = mod.ClassifyExample(examples[0], neighborList=neighborList)
    self.assertEqual(res, 1)
    self.assertEqual(neighborList[0][1], examples[0])

    self.assertEqual(mod.GetName(), '')
    mod.SetName('name')
    self.assertEqual(mod.GetName(), 'name')
    self.assertEqual(mod.type(), 'Classification Model')
    mod.NameModel('this argument is ignored')
    self.assertEqual(mod.GetName(), 'Classification Model')

  def test3Regress(self):
    # """ a carefully laid out regression data set where the results are clear: """
    fName = os.path.join(RDConfig.RDCodeDir, 'ML', 'KNN', 'test_data', 'sample_pts.csv')
    data = DataUtils.TextFileToData(fName)
    examples = data.GetNamedData()
    nvars = data.GetNVars()
    attrs = list(range(1, nvars + 1))
    numNeigh = 4
    metric = DistFunctions.EuclideanDist
    mdl = KNNRegressionModel.KNNRegressionModel(numNeigh, attrs, metric)
    mdl.SetTrainingExamples(examples)

    res = mdl.PredictExample([4, -3.5, 2.5, 0])
    assert feq(res, 1.25)
    res = mdl.PredictExample([4, 3, 2, 0])
    assert feq(res, 1.5)
    res = mdl.PredictExample([4, 3, -2.5, 0])
    assert feq(res, -1.5)
    # Use a distance dependent weight for the neighbours
    res = mdl.PredictExample([4, 3, -2.5, 0], weightedAverage=True)
    self.assertAlmostEqual(res, -1.6)
    # Check the case that the example is identical to one of the neighbours (distance = 0)
    neighborList = []
    res = mdl.PredictExample(examples[0], weightedAverage=True, neighborList=neighborList)
    self.assertAlmostEqual(res, 1.5857864)
    self.assertEqual(neighborList[0][1], examples[0])

    self.assertEqual(mdl.GetBadExamples(), [])

    self.assertEqual(mdl.GetName(), '')
    mdl.SetName('name')
    self.assertEqual(mdl.GetName(), 'name')
    self.assertEqual(mdl.type(), 'Regression Model')
    mdl.NameModel('this argument is ignored')
    self.assertEqual(mdl.GetName(), 'Regression Model')

    self.assertEqual(sorted(mdl.GetTrainingExamples() + mdl.GetTestExamples()), sorted(examples))

  def test4XValRegress(self):
    fName = os.path.join(RDConfig.RDCodeDir, 'ML', 'KNN', 'test_data', 'random_pts.csv')
    data = DataUtils.TextFileToData(fName)
    examples = data.GetNamedData()
    npvals = data.GetNPossibleVals()
    nvars = data.GetNVars()
    attrs = list(range(1, nvars + 1))
    numNeigh = 11
    _, err = CrossValidate.CrossValidationDriver(examples, attrs, npvals, numNeigh, silent=1,
                                                 modelBuilder=CrossValidate.makeRegressionModel)
    # NOTE: this number hasn't been extensively checked
    self.assertAlmostEqual(err, 0.0777, 4)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()
